###Libraries###
library(Rcpp)
library(RcppArmadillo)
library(RcppProgress)


#####################################################SDP function (recording second best behavioural decision)##############################################################################################################################################################################################################################################################################################################
sourceCpp(code='

#include <Rcpp.h>
#include <math.h>
using namespace std;
// [[Rcpp::depends(RcppProgress)]]
#include <progress.hpp>

int vecmax(Rcpp::NumericVector x) {     //function to return the position of the maximum element in a vector (in C++ indexing, i.e. starting at 0)
  Rcpp::NumericVector::iterator it = std::max_element(x.begin(), x.end());
  return it - x.begin();
}

int vec2max(Rcpp::NumericVector x) {     //function to return the position of the second to maximum element of a vector (this is in R indexing terms)
  Rcpp::NumericVector sorted = clone(x).sort();
  Rcpp::IntegerVector matched= match(sorted, x);
  return matched[matched.size()-2];
}

float roundTo2dec(float x)
 {
       Rcpp::NumericVector vec(1);
       vec[0]=x;
       vec=Rcpp::round(vec,2);
       return vec[0];
 }

float roundToThousandths(float x)
 {
     x /=1000;
       return floor(x + 0.5) * 1000;
 }

float roundToHundreds(float x)
 {
     x /=100;
       return floor(x + 0.5) * 100;
 }

Rcpp::NumericVector sequence(float minx, float maxx, float intx)
 {
     int n=(maxx-minx)/intx;
     Rcpp::NumericVector x(n+1);
     double total=minx;
     x[0]=total;
     for(int i = 1; i < (n+1); ++i) {
       total = total+intx;
       x[i] = total;
       }
     return x;
 }


float ag(int gtime, Rcpp::NumericVector Wc_d, float cost_kg, Rcpp::NumericVector propg, float Chg_tot)                                   //function to calculate gestational costs given stage of pregnancy in days from start
 {
 return Wc_d[gtime]*cost_kg + propg[gtime]*Chg_tot;
 }


float cot(float blubber, float lean, float scen)
 {
 float cost=7.79*pow(blubber + lean,-0.29)*scen;
 return cost;
 }

float blub_perc(float milk, int t,float blub_per, float me, float Er, float u, int t_il, Rcpp::NumericVector leand)
 {
           int day=t-t_il;
           float bm=leand[day]*blub_per/(1-blub_per);     //current blubber mass, derived from % of blubber and lean mass in this time step
           float Ed=milk*me;                              //energy received by the calf given amount of milk
           float S=Ed-Er;                                 //balance of energy usable to deposit blubber (or to cover costs of survival and growth) - it can be negative
           float D=S/u;                                   //kg of blubber that are deposited/used as the result of energy balance
           float nbm=D+bm;                                //new blubber mass
           float perc=nbm/(leand[day+1]+nbm);
           return perc;             //return new blubber %
 }


float extra_m(float nper, int t, int t_il, float mC, float E, Rcpp::NumericVector leand)     //calculates how much blubber mass the calf would receive above the maximum % blubber (so that it is summed back to mothers mass)
 {
           float pMax=leand[t-t_il+1]*mC/(1-mC);      //this is the maximum blubber mass the calf can have on the following day
           float extra;
           float nbm=leand[t-t_il+1]*nper/(1-nper);   //this is the blubber mass it would achieve based on the milk the mother can give
           if (nper>mC){
            extra=(nbm-pMax);
            }else{
            extra=0;
            }
           extra=extra/E;
           return extra; //returns extra mass (which is not delivered to the calf but retained by the female (with loss in energy transfer)
 }


Rcpp::NumericVector lr(Rcpp::NumericVector dens, Rcpp::NumericVector w)      //function to extract lunging rate given krill density
 {
    Rcpp::NumericVector wa(dens.size());
    Rcpp::LogicalVector lll=(dens<0.2);
    Rcpp::LogicalVector mmm=(dens>=0.2 & dens<=0.4);
    Rcpp::LogicalVector hhh=(dens>0.4);

    for (int i=0; i<dens.size(); i++){
     if(lll[i]) wa[i]=w[0];
     if(mmm[i]) wa[i]=w[1];
     if(hhh[i]) wa[i]=w[2];
     }
    return wa;
 }


Rcpp::NumericVector tf(Rcpp::NumericVector dens, Rcpp::NumericVector w, float beta, float psi, int tc, int l_i, int t, float for_hoursu)      //function to calculate time spent feeding (given patch density and anatomical limitations at given length)
 {
    Rcpp::NumericVector ti(dens.size());
    Rcpp::NumericVector ni(dens.size());
    Rcpp::NumericVector lunge=lr(dens,w);
    Rcpp::NumericVector hours(dens.size());
    Rcpp::NumericVector frate(dens.size());

    for (int i=0; i<dens.size(); i++){
     frate[i]=lunge[i]*beta*dens[i];                        //max amount of food per hour
     if(frate[i]!=0) ti[i]=psi/frate[i];                    //time (hours) to fill forestomach (depends on density; if density is 0, time is 0)
     if((ti[i]>tc) && (ti[i]!=0)) ni[i]=for_hoursu/ti[i];   //number of times it can fill the forestomach in a 12 hour period
     if((ti[i]<=tc) && (ti[i]!=0)) ni[i]=for_hoursu/tc;
     hours[i]=ti[i]*ni[i];
     }
    return hours;
 }


Rcpp::NumericVector cf(float blubber, float lean, float raamr, Rcpp::NumericVector dens, Rcpp::NumericVector w, float beta, float psi, int tc, int l_i, int t, float for_hoursu) //function to calculate cost of feeding
 {
    Rcpp::NumericVector cfeed(dens.size());
    Rcpp::NumericVector hours=tf(dens,w,beta,psi,tc,l_i,t,for_hoursu);
    for (int i=0; i<dens.size(); i++){
     cfeed[i]=hours[i]*60*60*raamr*(blubber+lean)/1000; //multiply RAAMR/Mc(W) by seconds and by kg, and divide by 1000 to get kJ
     }
    return cfeed;
 }


Rcpp::NumericVector cr(float blubber, float lean, Rcpp::NumericVector dens, Rcpp::NumericVector w, float beta, float psi, int tc, int l_i, int t, float for_hoursu, float scen)         //function to calculate cost of resting (given patch density to calculate time spent foraging, weight and anatomical limitations at given length)
 {
    float BMR=4*(pow(blubber + lean,0.75))*(24*60*60)/1000;  //kJ/d  (from Kleiber equation BMR(W)=4Mc^0.75: multiply by seconds and by kg and divide by 1000 to get kJ
    float BMRr=2*BMR*scen;                                   //MR while resting kJ/d (as in Potvin et al 2012)
    Rcpp::NumericVector crest(dens.size());
    Rcpp::NumericVector hours=tf(dens,w,beta,psi,tc,l_i,t,for_hoursu);
    for (int i=0; i<dens.size(); i++){
     crest[i]=(24-hours[i])/24*BMRr;
     }
    return crest;
 }


Rcpp::NumericVector food(Rcpp::NumericVector dens, Rcpp::NumericVector w, float beta, float psi, int tc, int l_i, int t, float for_hoursu, int rho, float A)               //function to calculate amount of energy from engulfed food (given patch density and anatomical limitations at given length)
 {
    Rcpp::NumericVector ti(dens.size());
    Rcpp::NumericVector ni(dens.size());
    Rcpp::NumericVector lunge=lr(dens,w);
    Rcpp::NumericVector fq(dens.size());
    Rcpp::NumericVector frate(dens.size());

    for (int i=0; i<dens.size(); i++){
     frate[i]=lunge[i]*beta*dens[i];                        //max amount of food per hour
     if(frate[i]!=0) ti[i]=psi/frate[i];                    //time (hours) to fill forestomach (depends on density; if density is 0, time is 0)
     if((ti[i]>tc) && (ti[i]!=0)) ni[i]=for_hoursu/ti[i];   //number of times it can fill the forestomach in a 12 hour period
     if((ti[i]<=tc) && (ti[i]!=0)) ni[i]=for_hoursu/tc;
     fq[i]=ni[i]*psi*rho*A;
     }
    return fq;
 }


//functions for interpolations

Rcpp::NumericVector interp_mx(Rcpp::NumericVector x1, Rcpp::NumericVector m1, float xc, float mc, Rcpp::NumericVector xs_l, Rcpp::NumericVector ms, int int_x, float int_m, int l_i, int i,Rcpp::NumericVector F1s,int locs)  //interpolation for lactating phase
 {
      Rcpp::NumericVector delta_x(x1.size());
      Rcpp::NumericVector delta_m(m1.size());
      Rcpp::NumericVector interpolm(x1.size());
      for(int j=0; j<x1.size(); j++){
         int x1t_i=(x1[j]-xc)/int_x;
         float xi=(xs_l.size()-1)/(max(xs_l)-xc)*(x1[j]-xc);
         delta_x[j]=xi-x1t_i;

         int m1t_i=(m1[j]-mc)/int_m;
         float mi=(ms.size()-1)/(max(ms)-mc)*(m1[j]-mc);
         delta_m[j]=mi-m1t_i;

         interpolm[j]=(1-delta_x[j])*(1-delta_m[j])*F1s[x1t_i*ms.size()+xs_l.size()*ms.size()*l_i+xs_l.size()*ms.size()*(locs+1)*i+m1t_i] +
         (1-delta_x[j])*delta_m[j]*F1s[x1t_i*ms.size()+xs_l.size()*ms.size()*l_i+xs_l.size()*ms.size()*(locs+1)*i+m1t_i+1]+
         delta_x[j]*(1-delta_m[j])*F1s[(x1t_i+1)*ms.size()+xs_l.size()*ms.size()*l_i+xs_l.size()*ms.size()*(locs+1)*i+m1t_i] +
         delta_x[j]*delta_m[j]*F1s[(x1t_i+1)*ms.size()+xs_l.size()*ms.size()*l_i+xs_l.size()*ms.size()*(locs+1)*i+m1t_i+1];
         }
      return interpolm;
 }



Rcpp::NumericVector interp_x(Rcpp::NumericVector x1, float x_abo, Rcpp::NumericVector xs_p, int int_x, int l_i,int i, Rcpp::NumericVector F1p,int locs)          //interpolation for pregnant phase
 {
      Rcpp::NumericVector delta_x(x1.size());
      Rcpp::NumericVector interpolm(x1.size());
      for(int j=0; j<x1.size(); j++){
         int x1t_i=(x1[j]-(x_abo-int_x))/int_x;
         float xi=(xs_p.size()-1)/(max(xs_p)-(x_abo-int_x))*(x1[j]-(x_abo-int_x));
         delta_x[j]=xi-x1t_i;

         interpolm[j]=(1-delta_x[j])*F1p[x1t_i*(locs+1)+(xs_p.size())*(locs+1)*i+l_i] +
         delta_x[j]*F1p[(x1t_i+1)*(locs+1)+(xs_p.size())*(locs+1)*i+l_i];
         }
      return interpolm;
 }




// [[Rcpp::export]]

Rcpp::List SDP(Rcpp::NumericVector yj, Rcpp::NumericVector pj, Rcpp::NumericVector for_hours, Rcpp::NumericVector lambda, Rcpp::NumericVector scenarion, bool display_progress=true) //main SDP function
{

int T, locs, feed_g, length, int_x, d, back, timeS, Mmax, tc, rho, gamma;
float weight, lean, max_perc, min_perc, death_perc, C, x_abo, x_lac, xc, mc, mC, int_m, cost_kg, Chg_tot, me, E, wb, lb, ww, lw, lgd, meat_cal, MRc, LEd, u, raamr, beta, psi, A, m50;
Rcpp::NumericVector xs_l, xs_p, gtime, ms, Wc, Wc_d(334), propg(334), Chg(334), leand(212), w(3);

T=454;     //max time steps (1 April - end of June of the following year; weaning after 7 months, i.e. 210 days after birth)
locs=36;   //number of locations
feed_g=15; //this is the location at which the main summer feeding ground starts (California current)

//morphometrics
length=22;                                      //from Gilpatrick and Perryman 2008, whales were on average smaller than in the Southern Ocean (22 m vs 25 m in the SO)
weight=4.6*pow(length,3.05)*scenarion[0];       //average weight of a whale of that length; length(m)-weight(kg) relationship  from Lockyer 1976
lean=roundToHundreds(0.73*scenarion[1]*weight); //average mass of lean tissue (73%)

//extremes for x:
max_perc=0.35;                             //maximum observed % of blubber in the body (Lockyer data 1976 and 1981)
min_perc=0.16;                             //minimum observed % of blubber in the body (Lockyer data 1976 and 1981)
death_perc=0.05;                           //% of blubber at which the whale dies
C=roundToThousandths(max_perc*lean/(1-max_perc)*scenarion[2]);        //max blubber mass a female can reach
x_abo=roundToThousandths(min_perc*lean/(1-min_perc)*scenarion[4]);    //minimum blubber mass to carry on pregnancy. If blubber falls below this threshold: abortion
x_lac=roundToThousandths(min_perc*lean/(1-min_perc)*scenarion[5]);    //minimum blubber mass to deliver milk. If blubber falls below this threshold: no lactation
xc=roundToThousandths(death_perc*lean/(1-death_perc)*scenarion[3]);   //at this blubber mass, the female dies (automatic abortion if pregnant, but lactating calf can survive)
int_x=500;

xs_l=sequence(xc,C+int_x,int_x);           //possible values of blubber mass. Upper limit needs to be C+int for interpolation (lactation)
xs_p=sequence(x_abo-int_x,C+int_x,int_x);  //possible values of blubber mass. Upper limit needs to be C+int for interpolation (pregnancy)

//extremes for m (% calf blubber mass):
mc=roundTo2dec(0.05*scenarion[7]);     //min % of blubber mass (at 5% blubber mass, calf dies)
mC=roundTo2dec(0.44*scenarion[6]);     //max % of blubber mass (calf cannot have more than 44% fat)
int_m=0.01; //for calf blubber bins (1%)
ms=sequence(mc,roundTo2dec(0.44*scenarion[6]+int_m),int_m);


//////Energetics functions//////

//movement/migration
d = 100;                                         //in km; distance they can cover in a day at optimal speed of 1.1 m/s (1.1*60*60*24/1000)
back = 2;                                        //location after which female has the option to go backwards

//pregnancy (gestation costs)
gtime=sequence(1,334,1);                         //gestation is 11 months
Wc=pow(0.52*(gtime-73),3)/1000*scenarion[14];    //foetal growth for blue whales in kg, where t is the number of days since conception. From Hugget and Widdas 1951 (in Lockyer 1986)
Wc[Wc<0.001]=0;                                  //set weight of foetus to 0 when less than 1g
cost_kg=2940*4.184*scenarion[13];                //kJ/kg of foetus weight. 2940 kCal/kg from Lockyer (1987 and 2007) (1kCal=4.184 kJ)
for(int i = 0; i < (Wc.size()-1); ++i) {
 Wc_d[i]=(Wc[i+1]-Wc[i]);
 propg[i]=(Wc[i+1]-Wc[i])/max(Wc);
 }
propg[Wc.size()-1]=NAN;
Chg_tot=4400*4.184*pow(max(Wc),1.2)*scenarion[15]; //total cost of heat of gestation (from Lockyer 2007, transformed to kJ)
Chg=propg*Chg_tot;                                 //heat of gestation (in kJ/day) depending on day of pregnancy (i.e on proportional growth on that day)
timeS=90;                                          //at the beginning of the modelled period (1 Apr), the female is 90 days pregnant

//lactation and calf growth
Mmax=220*scenarion[10];      //max milk delivered daily. I used the midpoint given by Oftedal 1997.
me=4137*4.184*scenarion[11]; //energy content (kJ/kg) of milk, 4137 kCal/kg (Lockyer 1986)
E=0.9*scenarion[12];         //90% mammary gland efficiency (Lockyer 2007)
wb=Wc[Wc.size()-1];          //weight at birth kg (from gestation growth curve)
lb=wb*0.94*scenarion[17];    //lean mass at birth (assuming calf is mostly lean at birth), but blubber % needs to be one interval above threshold for death
ww=17000*scenarion[18]+wb;   //avg estimated growth (Lockyer) + mass at birth (i.e. mass at weaning)
lw=ww*0.73*scenarion[19];    //lean mass at weaning we assume that the weaned calf has on average same % of blubber and lean mass as adult
lgd=(lw-lb)/211;             //calf avg daily growth in lean mass; kg/d

leand[0]=lb;
for(int i = 1; i < 212; ++i) {
 leand[i]= leand[i-1] + lgd; //total lean mass on each day, including the first day. Calf gets weaned after the last day of lactation (i.e. on day post-birth 211)
 }
meat_cal=1500*4.184*scenarion[20]; //kJ/kg, from kcal/kg value by Petrides and Swank 1965 and Brody 1968 in Lockyer 1981

LEd=lgd*meat_cal;                  //kJ required to grow each day
MRc=200000*4.184*scenarion[21];    //kJ to sustain maintainance/metabolic rate Lockyer 1981(1kCal=4.184 kJ)

float Er=MRc + LEd;                //total daily amount of energy required by the calf to survive and grow
int daysize=285*scenarion[16];     //threshold time for the calf to be able to travel outside the breeding ground with its mother

//feeding and resting
u=31798*scenarion[22];             //Utilizable energy (kJ/kg) in blubber (Lockyer 1981)
w[0]=10*scenarion[23];             //number of lunges per hour (Goldbogen et al. 2011 and 2012, and Hazen et al. 2015 - 2, 4, or 6 lunges per dive on average (depending on krill density: <0.2, 0.2-0.4, >0.4); dive is 9.8 min + 2.7 min at the surface, i.e. 12.5 min on average, i.e. 5 dives/hour = 5*c(2,4,6)=c(10,20,30))
w[1]=20*scenarion[24];             //number of lunges per hour (Goldbogen et al. 2011 and 2012, and Hazen et al. 2015 - 2, 4, or 6 lunges per dive on average (depending on krill density: <0.2, 0.2-0.4, >0.4); dive is 9.8 min + 2.7 min at the surface, i.e. 12.5 min on average, i.e. 5 dives/hour = 5*c(2,4,6)=c(10,20,30))
w[2]=30*scenarion[25];             //number of lunges per hour (Goldbogen et al. 2011 and 2012, and Hazen et al. 2015 - 2, 4, or 6 lunges per dive on average (depending on krill density: <0.2, 0.2-0.4, >0.4); dive is 9.8 min + 2.7 min at the surface, i.e. 12.5 min on average, i.e. 5 dives/hour = 5*c(2,4,6)=c(10,20,30))
raamr= 26.23*pow(length,-0.809)*scenarion[26];  //in W/kg; this is RAAMR/Mc
beta=0.0011*pow(length,3.56)*scenarion[27];     //volume of water ingested in a lunge
tc=4*scenarion[28];                             //number of hours to clear forestomach
psi=0.47*pow(length,2.88)*scenarion[29];        //forestomach capacity - adjusted from fin whales as in Weidenmann et al 2011
rho=4184*scenarion[30];                         //kJ/kg amount of energy in krill (Lockyer 1981 in Weidenmann et al 2011)
A=0.84*scenarion[31];                           //assimilation efficiency (Goldbogen et al 2011)


//////Terminal functions//////

//terminal function - pregnant
Rcpp::NumericVector F1p(xs_p.size()*(locs+1)*3);

//fill the last size x with -99
for(int i = 0; i < (locs+1); ++i) {
 for(int k = 0; k < 3; ++k) {
  F1p[(xs_p.size()-1)*(locs+1)+(xs_p.size())*(locs+1)*k+i]=(-99);
  }
 }

//fill all but the first size with 1
for(int j = 1; j < (xs_p.size()-1); ++j) {
 for(int i = 0; i < (locs+1); ++i) {
  for(int k = 0; k < 3; ++k) {
   F1p[j*(locs+1)+(xs_p.size())*(locs+1)*k+i]=1;
   }
  }
 }

//fill the last location with -99
for(int j = 0; j < (xs_p.size()-1); ++j) {
  for(int k = 0; k < 3; ++k) {
   F1p[j*(locs+1)+(xs_p.size())*(locs+1)*k+locs]=(-99);
   }
 }

//terminal function - lactating
gamma=8*scenarion[9];  //steepness of fitness function
m50=0.27*scenarion[8]; //calf survival probability is 50% when blubber % is the same as the observed one for adults
//initialise F1s as F(x,m,l,i,T), i.e. final values of Fs (0 if m<=mc, terminal function if m>mc)
Rcpp::NumericVector F1s(xs_l.size()*ms.size()*(locs+1)*3);

//fill the last size m with -99
for(int j = 0; j < (xs_l.size()); ++j) {
  for(int i = 0; i < (locs+1); ++i) {
   for(int k = 0; k < 3; ++k) {
    F1s[j*ms.size()+xs_l.size()*ms.size()*i+xs_l.size()*ms.size()*(locs+1)*k+(ms.size()-1)]=(-99);
    }
   }
 }

//fill in with terminal fitness for m
for(int j = 0; j < (xs_l.size()); ++j) {
 for(int mm = 1; mm < (ms.size()-1); ++mm) {
  for(int i = 0; i < (locs+1); ++i) {
   for(int k = 0; k < 3; ++k) {
    F1s[j*ms.size()+xs_l.size()*ms.size()*i+xs_l.size()*ms.size()*(locs+1)*k+mm]=pow(ms[mm],gamma)/(pow(ms[mm],gamma) + pow(m50,gamma));
    }
   }
  }
 }

//fill the last location with -99
for(int j = 0; j < (xs_l.size()); ++j) {
 for(int mm = 0; mm < (ms.size()); ++mm) {
   for(int k = 0; k < 3; ++k) {
    F1s[j*ms.size()+xs_l.size()*ms.size()*locs+xs_l.size()*ms.size()*(locs+1)*k+mm]=(-99);
    }
   }
 }

//fill the last size x with -99
 for(int mm = 0; mm < (ms.size()); ++mm) {
  for(int i = 0; i < (locs+1); ++i) {
   for(int k = 0; k < 3; ++k) {
    F1s[(xs_l.size()-1)*ms.size()+xs_l.size()*ms.size()*i+xs_l.size()*ms.size()*(locs+1)*k+mm]=(-99);
    }
   }
  }


//////Arrays to store results//////

//lactating females
Rcpp::NumericVector llt=sequence(244,T-1,1);
Rcpp::NumericVector resDl(xs_l.size()*ms.size()*(locs)*3*llt.size());
Rcpp::NumericVector resDl2(xs_l.size()*ms.size()*(locs)*3*llt.size());
Rcpp::NumericVector resF0l(xs_l.size()*ms.size()*(locs)*3*llt.size());
// initialize to all NaNs
for (int i=0; i<xs_l.size()*ms.size()*(locs)*3*llt.size(); ++i) {resDl[i] = NAN; resDl2[i] = NAN; resF0l[i] = NAN;}

int t_il=(T-1)-llt.size();

//pregnant females
Rcpp::NumericVector resDp(xs_p.size()*(locs)*3*t_il);
Rcpp::NumericVector resDp2(xs_p.size()*(locs)*3*t_il);
Rcpp::NumericVector resF0p(xs_p.size()*(locs)*3*t_il);
// initialize to all NaNs
for (int i=0; i<xs_p.size()*(locs)*3*t_il; ++i) {resDp[i] = NAN; resDp2[i] = NAN; resF0p[i] = NAN;}

Progress p(T-2, display_progress);

////////////////////////////////////SDP equations - backward iteration////////////////////////////////////////////

for (int t=(T-2); t>=0; t--){                        //cycle over time periods (backward, from T-2 to first time period, different from R because arrays start at 0

p.increment(); //update progress bar

//lactating and migrating to summer feeding grounds
 if(t>242){   //in C++ 243-1
  Rcpp::NumericVector F0((xs_l.size()-1)*(ms.size()-1)*(locs)*3);  //array F0 to store max fitness at t for different values of x, m, l and i
  Rcpp::NumericVector D((xs_l.size()-1)*(ms.size()-1)*(locs)*3);   //array D to store best strategy to choose depending on x, m, l and i
  Rcpp::NumericVector D2((xs_l.size()-1)*(ms.size()-1)*(locs)*3);  //array D2 to store second best strategy to choose depending on x, m, l and i

  //set fitness value to phi when x=xc (if mother dies, calf lives with probability that depends on current blubber mass - accounting for growth costs up to 7 months):
  float Erem=Er*(T-t)/u;                                           //blubber required to survive and grow up to 7 months (without milk from mother), i.e. for T-t days remaining, until when calf can autonomously feed
  Rcpp::NumericVector bm(ms.size()-1);
  Rcpp::NumericVector fms(ms.size()-1);
  for (int bmi=0; bmi<(ms.size()-1); ++bmi){
   bm[bmi]=leand[t-t_il]*ms[bmi]/(1-ms[bmi]);                      //current blubber mass, derived from % of blubber and lean mass in previous time step
   fms[bmi]=(bm[bmi]-Erem)/(leand[leand.size()-1]+(bm[bmi]-Erem)); //final % blubber mass, given current mass and mass required to survive and grow up to 7 months
   if(fms[bmi]<mc) fms[bmi]=mc;
   }

  for(int mm = 0; mm < (ms.size()-1); ++mm) {
    for(int ll = 0; ll < (locs); ++ll) {
     for(int ii = 0; ii < 3; ++ii) {
      F0[0*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*ll+(xs_l.size()-1)*(ms.size()-1)*(locs)*ii+mm]=pow(fms[mm],gamma)/(pow(fms[mm],gamma) + pow(m50,gamma));
      D[0*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*ll+(xs_l.size()-1)*(ms.size()-1)*(locs)*ii+mm]=0;
      }
     }
    }

  for(int ll = 0; ll < (locs); ++ll) {
     for(int ii = 0; ii < 3; ++ii) {
      F0[0*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*ll+(xs_l.size()-1)*(ms.size()-1)*(locs)*ii+0]=0; //set fitness to 0 when milk is mc
      }
     }

  for(int mm = 0; mm < (ms.size()-1); ++mm) {
    for(int ll = 0; ll < (locs); ++ll) {
     for(int ii = 0; ii < 3; ++ii) {
      resDl[0*ms.size()+xs_l.size()*ms.size()*ll+xs_l.size()*ms.size()*(locs)*ii+xs_l.size()*ms.size()*(locs)*3*(t-t_il)+mm]=D[0*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*ll+(xs_l.size()-1)*(ms.size()-1)*(locs)*ii+mm];
      resDl2[0*ms.size()+xs_l.size()*ms.size()*ll+xs_l.size()*ms.size()*(locs)*ii+xs_l.size()*ms.size()*(locs)*3*(t-t_il)+mm]=D[0*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*ll+(xs_l.size()-1)*(ms.size()-1)*(locs)*ii+mm];
      resF0l[0*ms.size()+xs_l.size()*ms.size()*ll+xs_l.size()*ms.size()*(locs)*ii+xs_l.size()*ms.size()*(locs)*3*(t-t_il)+mm]=F0[0*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*ll+(xs_l.size()-1)*(ms.size()-1)*(locs)*ii+mm];
      }
     }
    }

  //set fitness value to 0 when m=mc (i.e. m_i=0) (if calf blubber % is 0.05 or less, calf dies):
  for(int xx = 0; xx < (xs_l.size()-1); ++xx) {
    for(int ll = 0; ll < (locs); ++ll) {
     for(int ii = 0; ii < 3; ++ii) {
      F0[xx*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*ll+(xs_l.size()-1)*(ms.size()-1)*(locs)*ii+0]=0;
      D[xx*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*ll+(xs_l.size()-1)*(ms.size()-1)*(locs)*ii+0]=0;
      }
     }
    }

  for(int xx = 0; xx < (xs_l.size()-1); ++xx) {
    for(int ll = 0; ll < (locs); ++ll) {
     for(int ii = 0; ii < 3; ++ii) {
      resDl[xx*ms.size()+xs_l.size()*ms.size()*ll+xs_l.size()*ms.size()*(locs)*ii+xs_l.size()*ms.size()*(locs)*3*(t-t_il)+0]=D[xx*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*ll+(xs_l.size()-1)*(ms.size()-1)*(locs)*ii+0];
      resDl2[xx*ms.size()+xs_l.size()*ms.size()*ll+xs_l.size()*ms.size()*(locs)*ii+xs_l.size()*ms.size()*(locs)*3*(t-t_il)+0]=D[xx*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*ll+(xs_l.size()-1)*(ms.size()-1)*(locs)*ii+0];
      resF0l[xx*ms.size()+xs_l.size()*ms.size()*ll+xs_l.size()*ms.size()*(locs)*ii+xs_l.size()*ms.size()*(locs)*3*(t-t_il)+0]=F0[xx*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*ll+(xs_l.size()-1)*(ms.size()-1)*(locs)*ii+0];
      }
     }
    }

 for (int x_i=1; x_i<(xs_l.size()-1); ++x_i){                    //cycle over all possible values of x (when x_i=0 she is dead, so milk doesnt change and F1s stays fixed at original values)
  for (int m_i=1; m_i<(ms.size()-1); ++m_i){                     //cycle over all possible values of m (calf % blubber mass)

     int l_i=0;                                                  //when in the Gulf of California
     resDl[x_i*ms.size()+xs_l.size()*ms.size()*l_i+xs_l.size()*ms.size()*(locs)*1+xs_l.size()*ms.size()*(locs)*3*(t-t_il)+m_i]=0;
     resDl2[x_i*ms.size()+xs_l.size()*ms.size()*l_i+xs_l.size()*ms.size()*(locs)*1+xs_l.size()*ms.size()*(locs)*3*(t-t_il)+m_i]=0;
     resF0l[x_i*ms.size()+xs_l.size()*ms.size()*l_i+xs_l.size()*ms.size()*(locs)*1+xs_l.size()*ms.size()*(locs)*3*(t-t_il)+m_i]=NAN;
     Rcpp::NumericVector patches(2);
     patches[0]=0;
     patches[1]=2;
     for (int in=0; in < patches.size(); ++in){
      int i = patches[in];
      Rcpp::NumericVector x1;
      float x2;
      Rcpp::NumericVector yju(30);
      Rcpp::NumericVector pju(30);
      for (int yjj=0; yjj<30; ++yjj){
       yju[yjj]=yj[i*30 + 3*30*l_i + 3*30*(locs+1)*t + yjj];
       pju[yjj]=pj[i*30 + 3*30*l_i + 3*30*(locs+1)*t + yjj];
       }
      float for_hoursu;
      for_hoursu=for_hours[l_i*(T+1)+t];

      x1 = xs_l[x_i]+(-cf(xs_l[x_i], lean, raamr, yju, w, beta, psi, tc, l_i, t, for_hoursu) - cr(xs_l[x_i], lean, yju, w, beta, psi, tc, l_i, t, for_hoursu, scenarion[38]) + food(yju, w, beta, psi, tc, l_i, t, for_hoursu, rho, A))/u;                                          //feeding, resting and not delivering milk
      Rcpp::NumericVector test11=xs_l[x_i]+ (- cf(xs_l[x_i], lean, raamr, yju, w, beta, psi, tc, l_i, t, for_hoursu) - cr(xs_l[x_i], lean, yju, w, beta, psi, tc, l_i, t, for_hoursu, scenarion[38]) + food(yju, w, beta, psi, tc, l_i, t, for_hoursu, rho, A))/u - x_lac;          //if this is >0, she will deliver milk
      Rcpp::NumericVector test21=(xs_l[x_i]-x_lac)*u - cf(xs_l[x_i], lean, raamr, yju, w, beta, psi, tc, l_i, t, for_hoursu) - cr(xs_l[x_i], lean, yju, w, beta, psi, tc, l_i, t, for_hoursu, scenarion[38]) + food(yju, w, beta, psi, tc, l_i, t, for_hoursu, rho, A) - Mmax*me/E; //if this is >0, she will deliver Mmax milk
      Rcpp::NumericVector milk1(x1.size());
      Rcpp::NumericVector m1(x1.size());
      Rcpp::NumericVector F11(x1.size());
      for(int xx1=0; xx1<x1.size(); ++xx1){
       if((test11[xx1]>0) && (test21[xx1]<0)) x1[xx1]=x_lac;
       if((test11[xx1]>0) && (test21[xx1]>=0)) x1[xx1]=(xs_l[x_i]+ (- cf(xs_l[x_i], lean, raamr, yju, w, beta, psi, tc, l_i, t, for_hoursu) - cr(xs_l[x_i], lean, yju, w, beta, psi, tc, l_i, t, for_hoursu, scenarion[38]) + food(yju, w, beta, psi, tc, l_i, t, for_hoursu, rho, A) - Mmax*me/E)/u)[xx1];
       if(x1[xx1]<xc) x1[xx1]=xc;                         //truncated so that min is xc
       if(x1[xx1]>C) x1[xx1]=C;                           //truncated so that max is C
       if((test11[xx1]>0) && (test21[xx1]<0)) milk1[xx1] = (((xs_l[x_i]-x_lac)*u - cf(xs_l[x_i], lean, raamr, yju, w, beta, psi, tc, l_i, t, for_hoursu) - cr(xs_l[x_i], lean, yju, w, beta, psi, tc, l_i, t, for_hoursu, scenarion[38]) + food(yju, w, beta, psi, tc, l_i, t, for_hoursu, rho, A))*E/me)[xx1];
       if((test11[xx1]>0) && (test21[xx1]>=0)) milk1[xx1] = Mmax;
       m1[xx1]=blub_perc(milk1[xx1],t,ms[m_i],me,Er,u,t_il,leand);                  //calculates new % calf blubber mass (after milk delivery)
       x1<-x1+extra_m(m1[xx1],t,t_il,mC,E,leand);                                   //add to females blubber mass the mass she cannot deliver to the calf because calf is already at maximum blubber
       if(x1[xx1]<xc) x1[xx1]=xc;                         //truncated so that min is xc
       if(x1[xx1]>C) x1[xx1]=C;                           //truncated so that max is C
       if(m1[xx1]<mc) m1[xx1]=mc;                         //truncated so that min is mc
       if(m1[xx1]>mC) m1[xx1]=mC;                         //truncated so that max is mC
       }
      F11=interp_mx(x1,m1,xc,mc,xs_l,ms,int_x,int_m,l_i,i,F1s,locs);                 //calculates corresponding fitness

      x2 = xs_l[x_i] - cot(xs_l[x_i],lean, scenarion[39])*d*(xs_l[x_i]+lean)/u;                     //moving out of the Gulf and not delivering milk
      float test12 = xs_l[x_i] - cot(xs_l[x_i],lean, scenarion[39])*d*(xs_l[x_i]+lean)/u - x_lac;              //if this is >0, she will deliver milk
      float test22 = (xs_l[x_i]-x_lac)*u - cot(xs_l[x_i],lean, scenarion[39])*d*(xs_l[x_i]+lean) - Mmax*me/E;  //if this is >0, she will deliver Mmax milk
      if((test12>0) && (test22<0)) x2=x_lac;
      if((test12>0) && (test22>=0)) x2 = (xs_l[x_i] + (- cot(xs_l[x_i],lean, scenarion[39])*d*(xs_l[x_i]+lean) - Mmax*me/E)/u);
      if(x2<xc) x2=xc;                         //truncated so that min is xc
      if(x2>C) x2=C;                           //truncated so that max is C
      float milk2=0;
      if((test12>0) && (test22<0)) milk2= (((xs_l[x_i]-x_lac)*u - cot(xs_l[x_i],lean, scenarion[39])*d*(xs_l[x_i]+lean))*E/me);
      if((test12>0) && (test22>=0)) milk2= Mmax;
      float m2=blub_perc(milk2,t,ms[m_i],me,Er,u,t_il,leand);
      x2<-x2+extra_m(m2,t,t_il,mC,E,leand);                            //add to females blubber mass the mass she cannot deliver to the calf because calf is already at maximum blubber
      if(x2<xc) x2=xc;                         //truncated so that min is xc
      if(x2>C) x2=C;                           //truncated so that max is C
      if(m2<mc) m2=mc;                         //truncated so that min is xc
      if(m2>mC) m2=mC;                         //truncated so that max is C
      Rcpp::NumericVector F12(3);
      Rcpp::NumericVector x2v(1);
      Rcpp::NumericVector m2v(1);
      x2v[0]=x2;
      m2v[0]=m2;
      for (int iii=0;iii<3;++iii){
       Rcpp::NumericVector tmp2=interp_mx(x2v,m2v,xc,mc,xs_l,ms,int_x,int_m,l_i+1,iii,F1s,locs);
       F12[iii]=tmp2[0];
       }
      Rcpp::NumericVector lambda_f2(3);
      for (int la=0; la<3; ++la){
       lambda_f2[la]=lambda[(l_i+1)*3 + (locs+1)*3*(t+1) + la];          //lambda depends on location and time
       }

      Rcpp::NumericVector F13(3);
      for (int iii=0;iii<3;++iii){
       Rcpp::NumericVector tmp3=interp_mx(x2v,m2v,xc,mc,xs_l,ms,int_x,int_m,l_i,iii,F1s,locs);
       F13[iii]=tmp3[0];
       }
      Rcpp::NumericVector lambda_f3(3);
      for (int la=0; la<3; ++la){
       lambda_f3[la]=lambda[(l_i)*3 + (locs+1)*3*(t+1) + la];            //lambda depends on location and time
       }

      Rcpp::NumericVector V(3);
      V[0]=sum(pju*F11);
      V[1]=sum(lambda_f2*F12);
      V[2]=sum(lambda_f3*F13);
      if(t<daysize) V[1]=(-9999);                                       //the newborn calf is not able to move outside breeding ground for the first period
      F0[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=max(V);  //store max fitness
      V[0]=roundf(V[0]*pow(10,14))/pow(10,14);
      V[1]=roundf(V[1]*pow(10,14))/pow(10,14);
      V[2]=roundf(V[2]*pow(10,14))/pow(10,14);
      if ((V[0]==V[1])&&(V[1]==V[2])){
       D[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=4;
       }
       else
       {
       if((V[0]==V[1])&&(V[1]>V[2])){
        D[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=5;
        }
        else
        {
        if((V[0]==V[2])&&(V[2]>V[1])){
         D[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=6;
         }
         else
         {
         if((V[1]==V[2])&&(V[2]>V[0])){
          D[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=7;
          }
          else
          {
          D[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=(vecmax(V)+1);   //need to add 1 because its C++
          }
         }
        }
       }
      D2[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=vec2max(V);   //no need to add 1 because this function operates in R terms

      resDl[x_i*ms.size()+xs_l.size()*ms.size()*l_i+xs_l.size()*ms.size()*(locs)*i+xs_l.size()*ms.size()*(locs)*3*(t-t_il)+m_i]=D[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i];
      resDl2[x_i*ms.size()+xs_l.size()*ms.size()*l_i+xs_l.size()*ms.size()*(locs)*i+xs_l.size()*ms.size()*(locs)*3*(t-t_il)+m_i]=D2[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i];
      resF0l[x_i*ms.size()+xs_l.size()*ms.size()*l_i+xs_l.size()*ms.size()*(locs)*i+xs_l.size()*ms.size()*(locs)*3*(t-t_il)+m_i]=F0[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i];
      }

     l_i=1;                                           //when in the second breeding ground
     resDl[x_i*ms.size()+xs_l.size()*ms.size()*l_i+xs_l.size()*ms.size()*(locs)*1+xs_l.size()*ms.size()*(locs)*3*(t-t_il)+m_i]=0;
     resDl2[x_i*ms.size()+xs_l.size()*ms.size()*l_i+xs_l.size()*ms.size()*(locs)*1+xs_l.size()*ms.size()*(locs)*3*(t-t_il)+m_i]=0;
     resF0l[x_i*ms.size()+xs_l.size()*ms.size()*l_i+xs_l.size()*ms.size()*(locs)*1+xs_l.size()*ms.size()*(locs)*3*(t-t_il)+m_i]=NAN;
     for (int in=0; in < patches.size(); ++in){       //cycle over other two patch types
      int i = patches[in];
      Rcpp::NumericVector x1;
      float x2;
      Rcpp::NumericVector yju(30);
      Rcpp::NumericVector pju(30);
      for (int yjj=0; yjj<30; ++yjj){
       yju[yjj]=yj[i*30 + 3*30*l_i + 3*30*(locs+1)*t + yjj];
       pju[yjj]=pj[i*30 + 3*30*l_i + 3*30*(locs+1)*t + yjj];
       }
      float for_hoursu;
      for_hoursu=for_hours[l_i*(T+1)+t];

      x1 = xs_l[x_i]+(-cf(xs_l[x_i], lean, raamr, yju, w, beta, psi, tc, l_i, t, for_hoursu) - cr(xs_l[x_i], lean, yju, w, beta, psi, tc, l_i, t, for_hoursu, scenarion[38]) + food(yju, w, beta, psi, tc, l_i, t, for_hoursu, rho, A))/u;                    //feeding, resting and not delivering milk
      Rcpp::NumericVector test11=xs_l[x_i]+ (- cf(xs_l[x_i], lean, raamr, yju, w, beta, psi, tc, l_i, t, for_hoursu) - cr(xs_l[x_i], lean, yju, w, beta, psi, tc, l_i, t, for_hoursu, scenarion[38]) + food(yju, w, beta, psi, tc, l_i, t, for_hoursu, rho, A))/u - x_lac;           //if this is >0, she will deliver milk
      Rcpp::NumericVector test21=(xs_l[x_i]-x_lac)*u - cf(xs_l[x_i], lean, raamr, yju, w, beta, psi, tc, l_i, t, for_hoursu) - cr(xs_l[x_i], lean, yju, w, beta, psi, tc, l_i, t, for_hoursu, scenarion[38]) + food(yju, w, beta, psi, tc, l_i, t, for_hoursu, rho, A) - Mmax*me/E;  //if this is >0, she will deliver Mmax milk
      Rcpp::NumericVector milk1(x1.size());
      Rcpp::NumericVector m1(x1.size());
      Rcpp::NumericVector F11(x1.size());
      for(int xx1=0; xx1<x1.size(); ++xx1){
       if((test11[xx1]>0) && (test21[xx1]<0)) x1[xx1]=x_lac;
       if((test11[xx1]>0) && (test21[xx1]>=0)) x1[xx1]=(xs_l[x_i]+ (- cf(xs_l[x_i], lean, raamr, yju, w, beta, psi, tc, l_i, t, for_hoursu) - cr(xs_l[x_i], lean, yju, w, beta, psi, tc, l_i, t, for_hoursu, scenarion[38]) + food(yju, w, beta, psi, tc, l_i, t, for_hoursu, rho, A) - Mmax*me/E)/u)[xx1];
       if(x1[xx1]<xc) x1[xx1]=xc;                         //truncated so that min is xc
       if(x1[xx1]>C) x1[xx1]=C;                           //truncated so that max is C
       if((test11[xx1]>0) && (test21[xx1]<0)) milk1[xx1] = (((xs_l[x_i]-x_lac)*u - cf(xs_l[x_i], lean, raamr, yju, w, beta, psi, tc, l_i, t, for_hoursu) - cr(xs_l[x_i], lean, yju, w, beta, psi, tc, l_i, t, for_hoursu, scenarion[38]) + food(yju, w, beta, psi, tc, l_i, t, for_hoursu, rho, A))*E/me)[xx1];
       if((test11[xx1]>0) && (test21[xx1]>=0)) milk1[xx1] = Mmax;
       m1[xx1]=blub_perc(milk1[xx1],t,ms[m_i],me,Er,u,t_il,leand);           //calculates new % calf blubber mass (after milk delivery)
       x1<-x1+extra_m(m1[xx1],t,t_il,mC,E,leand);         //add to females blubber mass the mass she cannot deliver to the calf because calf is already at maximum blubber
       if(x1[xx1]<xc) x1[xx1]=xc;                         //truncated so that min is xc
       if(x1[xx1]>C) x1[xx1]=C;                           //truncated so that max is C
       if(m1[xx1]<mc) m1[xx1]=mc;                         //truncated so that min is mc
       if(m1[xx1]>mC) m1[xx1]=mC;                         //truncated so that max is mC
       }
      F11=interp_mx(x1,m1,xc,mc,xs_l,ms,int_x,int_m,l_i,i,F1s,locs);                 //calculates corresponding fitness

      x2 = xs_l[x_i] - cot(xs_l[x_i],lean, scenarion[39])*d*(xs_l[x_i]+lean)/u;                     //moving out of the Gulf and not delivering milk
      float test12 = xs_l[x_i] - cot(xs_l[x_i],lean, scenarion[39])*d*(xs_l[x_i]+lean)/u - x_lac;              //if this is >0, she will deliver milk
      float test22 = (xs_l[x_i]-x_lac)*u - cot(xs_l[x_i],lean, scenarion[39])*d*(xs_l[x_i]+lean) - Mmax*me/E;  //if this is >0, she will deliver Mmax milk
      if((test12>0) && (test22<0)) x2=x_lac;
      if((test12>0) && (test22>=0)) x2 = (xs_l[x_i] + (- cot(xs_l[x_i],lean, scenarion[39])*d*(xs_l[x_i]+lean) - Mmax*me/E)/u);
      if(x2<xc) x2=xc;                         //truncated so that min is xc
      if(x2>C) x2=C;                           //truncated so that max is C
      float milk2=0;
      if((test12>0) && (test22<0)) milk2= (((xs_l[x_i]-x_lac)*u - cot(xs_l[x_i],lean, scenarion[39])*d*(xs_l[x_i]+lean))*E/me);
      if((test12>0) && (test22>=0)) milk2= Mmax;
      float m2=blub_perc(milk2,t,ms[m_i],me,Er,u,t_il,leand);
      x2<-x2+extra_m(m2,t,t_il,mC,E,leand);    //add to females blubber mass the mass she cannot deliver to the calf because calf is already at maximum blubber
      if(x2<xc) x2=xc;                         //truncated so that min is xc
      if(x2>C) x2=C;                           //truncated so that max is C
      if(m2<mc) m2=mc;                         //truncated so that min is xc
      if(m2>mC) m2=mC;                         //truncated so that max is C
      Rcpp::NumericVector F12(3);
      Rcpp::NumericVector x2v(1);
      Rcpp::NumericVector m2v(1);
      x2v[0]=x2;
      m2v[0]=m2;
      for (int iii=0;iii<3;++iii){
       Rcpp::NumericVector tmp2=interp_mx(x2v,m2v,xc,mc,xs_l,ms,int_x,int_m,l_i+1,iii,F1s,locs);
       F12[iii]=tmp2[0];
       }
      Rcpp::NumericVector lambda_f2(3);
      for (int la=0; la<3; ++la){
       lambda_f2[la]=lambda[(l_i+1)*3 + (locs+1)*3*(t+1) + la];                 //lambda depends on location and time
       }

      Rcpp::NumericVector F13(3);
      for (int iii=0;iii<3;++iii){
       Rcpp::NumericVector tmp3=interp_mx(x2v,m2v,xc,mc,xs_l,ms,int_x,int_m,l_i-1,iii,F1s,locs);
       F13[iii]=tmp3[0];
       }
      Rcpp::NumericVector lambda_f3(3);
      for (int la=0; la<3; ++la){
       lambda_f3[la]=lambda[(l_i-1)*3 + (locs+1)*3*(t+1) + la];                 //lambda depends on location and time
       }

      Rcpp::NumericVector F14(3);
      for (int iii=0;iii<3;++iii){
       Rcpp::NumericVector tmp4=interp_mx(x2v,m2v,xc,mc,xs_l,ms,int_x,int_m,l_i,iii,F1s,locs);
       F14[iii]=tmp4[0];
       }
      Rcpp::NumericVector lambda_f4(3);
      for (int la=0; la<3; ++la){
       lambda_f4[la]=lambda[(l_i)*3 + (locs+1)*3*(t+1) + la];                   //lambda depends on location and time
       }

      Rcpp::NumericVector V2(4);
      V2[0]=sum(pju*F11);
      V2[1]=sum(lambda_f2*F12);
      V2[2]=sum(lambda_f3*F13);
      V2[3]=sum(lambda_f4*F14);
      if(t<daysize) V2[1]=(-9999);
      F0[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=max(V2); //store max fitness
      V2[0]=roundf(V2[0]*pow(10,14))/pow(10,14);
      V2[1]=roundf(V2[1]*pow(10,14))/pow(10,14);
      V2[2]=roundf(V2[2]*pow(10,14))/pow(10,14);
      V2[3]=roundf(V2[3]*pow(10,14))/pow(10,14);
      if ((V2[0]==V2[1])&&(V2[1]==V2[2])&&(V2[2]==V2[3])){
       D[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=5;
       }
       else
       {
       if((V2[0]==V2[1])&&(V2[1]==V2[2])&&(V2[2]>V2[3])){
        D[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=6;
        }
        else
        {
        if((V2[0]==V2[1])&&(V2[1]==V2[3])&&(V2[3]>V2[2])){
         D[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=7;
         }
         else
         {
         if((V2[0]==V2[2])&&(V2[2]==V2[3])&&(V2[3]>V2[1])){
          D[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=8;
          }
          else
          {
          if((V2[1]==V2[2])&&(V2[2]==V2[3])&&(V2[3]>V2[0])){
           D[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=9;
           }
           else
           {
           if((V2[0]==V2[2])&&(V2[2]==max(V2))&&(V2[2]>V2[3])){
            D[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=10;
            }
            else
            {
            if((V2[0]==V2[1])&&(V2[1]==max(V2))&&(V2[1]>V2[3])){
             D[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=11;
             }
             else
             {
             if((V2[0]==V2[3])&&(V2[3]==max(V2))&&(V2[3]>V2[1])){
              D[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=12;
              }
              else
              {
              if((V2[1]==V2[2])&&(V2[2]==max(V2))&&(V2[2]>V2[0])){
               D[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=13;
               }
               else
               {
               if((V2[1]==V2[3])&&(V2[3]==max(V2))&&(V2[3]>V2[0])){
                D[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=14;
                }
                else
                {
                if((V2[2]==V2[3])&&(V2[3]==max(V2))&&(V2[3]>V2[0])){
                 D[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=15;
                 }
                 else
                 {
                 D[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=(vecmax(V2)+1);
                 }
                }
               }
              }
             }
            }
           }
          }
         }
        }
       }
      D2[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=vec2max(V2);

      resDl[x_i*ms.size()+xs_l.size()*ms.size()*l_i+xs_l.size()*ms.size()*(locs)*i+xs_l.size()*ms.size()*(locs)*3*(t-t_il)+m_i]=D[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i];
      resDl2[x_i*ms.size()+xs_l.size()*ms.size()*l_i+xs_l.size()*ms.size()*(locs)*i+xs_l.size()*ms.size()*(locs)*3*(t-t_il)+m_i]=D2[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i];
      resF0l[x_i*ms.size()+xs_l.size()*ms.size()*l_i+xs_l.size()*ms.size()*(locs)*i+xs_l.size()*ms.size()*(locs)*3*(t-t_il)+m_i]=F0[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i];
      }

    for (int l_i=2;l_i<locs; ++l_i){                           //cycle over all other locations
     for (int i=0; i<3; ++i){                                  //cycle over all patch types
      Rcpp::NumericVector x1;
      float x2;
      Rcpp::NumericVector yju(30);
      Rcpp::NumericVector pju(30);
      for (int yjj=0; yjj<30; ++yjj){
       yju[yjj]=yj[i*30 + 3*30*l_i + 3*30*(locs+1)*t + yjj];
       pju[yjj]=pj[i*30 + 3*30*l_i + 3*30*(locs+1)*t + yjj];
       }
      float for_hoursu;
      for_hoursu=for_hours[l_i*(T+1)+t];

      x1 = xs_l[x_i]+(-cf(xs_l[x_i], lean, raamr, yju, w, beta, psi, tc, l_i, t, for_hoursu) - cr(xs_l[x_i], lean, yju, w, beta, psi, tc, l_i, t, for_hoursu, scenarion[38]) + food(yju, w, beta, psi, tc, l_i, t, for_hoursu, rho, A))/u;                    //feeding, resting and not delivering milk
      Rcpp::NumericVector test11=xs_l[x_i]+ (- cf(xs_l[x_i], lean, raamr, yju, w, beta, psi, tc, l_i, t, for_hoursu) - cr(xs_l[x_i], lean, yju, w, beta, psi, tc, l_i, t, for_hoursu, scenarion[38]) + food(yju, w, beta, psi, tc, l_i, t, for_hoursu, rho, A))/u - x_lac;         //if this is >0, she will deliver milk
      Rcpp::NumericVector test21=(xs_l[x_i]-x_lac)*u - cf(xs_l[x_i], lean, raamr, yju, w, beta, psi, tc, l_i, t, for_hoursu) - cr(xs_l[x_i], lean, yju, w, beta, psi, tc, l_i, t, for_hoursu, scenarion[38]) + food(yju, w, beta, psi, tc, l_i, t, for_hoursu, rho, A) - Mmax*me/E;  //if this is >0, she will deliver Mmax milk
      Rcpp::NumericVector milk1(x1.size());
      Rcpp::NumericVector m1(x1.size());
      Rcpp::NumericVector F11(x1.size());
      for(int xx1=0; xx1<x1.size(); ++xx1){
       if((test11[xx1]>0) && (test21[xx1]<0)) x1[xx1]=x_lac;
       if((test11[xx1]>0) && (test21[xx1]>=0)) x1[xx1]=(xs_l[x_i]+ (- cf(xs_l[x_i], lean, raamr, yju, w, beta, psi, tc, l_i, t, for_hoursu) - cr(xs_l[x_i], lean, yju, w, beta, psi, tc, l_i, t, for_hoursu, scenarion[38]) + food(yju, w, beta, psi, tc, l_i, t, for_hoursu, rho, A) - Mmax*me/E)/u)[xx1];
       if(x1[xx1]<xc) x1[xx1]=xc;                         //truncated so that min is xc
       if(x1[xx1]>C) x1[xx1]=C;                           //truncated so that max is C
       if((test11[xx1]>0) && (test21[xx1]<0)) milk1[xx1] = (((xs_l[x_i]-x_lac)*u - cf(xs_l[x_i], lean, raamr, yju, w, beta, psi, tc, l_i, t, for_hoursu) - cr(xs_l[x_i], lean, yju, w, beta, psi, tc, l_i, t, for_hoursu, scenarion[38]) + food(yju, w, beta, psi, tc, l_i, t, for_hoursu, rho, A))*E/me)[xx1];
       if((test11[xx1]>0) && (test21[xx1]>=0)) milk1[xx1] = Mmax;
       m1[xx1]=blub_perc(milk1[xx1],t,ms[m_i],me,Er,u,t_il,leand);           //calculates new % calf blubber mass (after milk delivery)
       x1<-x1+extra_m(m1[xx1],t,t_il,mC,E,leand);                            //add to females blubber mass the mass she cannot deliver to the calf because calf is already at maximum blubber
       if(x1[xx1]<xc) x1[xx1]=xc;                         //truncated so that min is xc
       if(x1[xx1]>C) x1[xx1]=C;                           //truncated so that max is C
       if(m1[xx1]<mc) m1[xx1]=mc;                         //truncated so that min is mc
       if(m1[xx1]>mC) m1[xx1]=mC;                         //truncated so that max is mC
       }
      F11=interp_mx(x1,m1,xc,mc,xs_l,ms,int_x,int_m,l_i,i,F1s,locs);                 //calculates corresponding fitness

      x2 = xs_l[x_i] - cot(xs_l[x_i],lean, scenarion[39])*d*(xs_l[x_i]+lean)/u;                     //moving out of the Gulf and not delivering milk
      float test12 = xs_l[x_i] - cot(xs_l[x_i],lean, scenarion[39])*d*(xs_l[x_i]+lean)/u - x_lac;              //if this is >0, she will deliver milk
      float test22 = (xs_l[x_i]-x_lac)*u - cot(xs_l[x_i],lean, scenarion[39])*d*(xs_l[x_i]+lean) - Mmax*me/E;  //if this is >0, she will deliver Mmax milk
      if((test12>0) && (test22<0)) x2=x_lac;
      if((test12>0) && (test22>=0)) x2 = (xs_l[x_i] + (- cot(xs_l[x_i],lean, scenarion[39])*d*(xs_l[x_i]+lean) - Mmax*me/E)/u);
      if(x2<xc) x2=xc;                         //truncated so that min is xc
      if(x2>C) x2=C;                           //truncated so that max is C
      float milk2=0;
      if((test12>0) && (test22<0)) milk2= (((xs_l[x_i]-x_lac)*u - cot(xs_l[x_i],lean, scenarion[39])*d*(xs_l[x_i]+lean))*E/me);
      if((test12>0) && (test22>=0)) milk2= Mmax;
      float m2=blub_perc(milk2,t,ms[m_i],me,Er,u,t_il,leand);
      x2<-x2+extra_m(m2,t,t_il,mC,E,leand);    //add to females blubber mass the mass she cannot deliver to the calf because calf is already at maximum blubber
      if(x2<xc) x2=xc;                         //truncated so that min is xc
      if(x2>C) x2=C;                           //truncated so that max is C
      if(m2<mc) m2=mc;                         //truncated so that min is xc
      if(m2>mC) m2=mC;                         //truncated so that max is C
      Rcpp::NumericVector F12(3);
      Rcpp::NumericVector x2v(1);
      Rcpp::NumericVector m2v(1);
      x2v[0]=x2;
      m2v[0]=m2;
      for (int iii=0;iii<3;++iii){
       Rcpp::NumericVector tmp2=interp_mx(x2v,m2v,xc,mc,xs_l,ms,int_x,int_m,l_i+1,iii,F1s,locs);
       F12[iii]=tmp2[0];
       }
      Rcpp::NumericVector lambda_f2(3);
      for (int la=0; la<3; ++la){
       lambda_f2[la]=lambda[(l_i+1)*3 + (locs+1)*3*(t+1) + la];                 //lambda depends on location and time
       }

      Rcpp::NumericVector F13(3);
      for (int iii=0;iii<3;++iii){
       Rcpp::NumericVector tmp3=interp_mx(x2v,m2v,xc,mc,xs_l,ms,int_x,int_m,l_i-1,iii,F1s,locs);
       F13[iii]=tmp3[0];
       }
      Rcpp::NumericVector lambda_f3(3);
      for (int la=0; la<3; ++la){
       lambda_f3[la]=lambda[(l_i-1)*3 + (locs+1)*3*(t+1) + la];                 //lambda depends on location and time
       }

      Rcpp::NumericVector V3(3);
      V3[0]=sum(pju*F11);
      V3[1]=sum(lambda_f2*F12);
      V3[2]=sum(lambda_f3*F13);
      F0[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=max(V3);                                         //store max fitness
      V3[0]=roundf(V3[0]*pow(10,14))/pow(10,14);
      V3[1]=roundf(V3[1]*pow(10,14))/pow(10,14);
      V3[2]=roundf(V3[2]*pow(10,14))/pow(10,14);
      if ((V3[0]==V3[1])&&(V3[1]==V3[2])){
       D[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=4;
       }
       else
       {
       if((V3[0]==V3[1])&&(V3[1]>V3[2])){
        D[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=5;
        }
        else
        {
        if((V3[0]==V3[2])&&(V3[2]>V3[1])){
         D[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=6;
         }
         else
         {
         if((V3[1]==V3[2])&&(V3[2]>V3[0])){
          D[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=7;
          }
          else
          {
          D[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=(vecmax(V3)+1);   //need to add 1 because its C++
          }
         }
        }
       }
      D2[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i]=vec2max(V3);   //no need to add 1 here

      resDl[x_i*ms.size()+xs_l.size()*ms.size()*l_i+xs_l.size()*ms.size()*(locs)*i+xs_l.size()*ms.size()*(locs)*3*(t-t_il)+m_i]=D[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i];
      resDl2[x_i*ms.size()+xs_l.size()*ms.size()*l_i+xs_l.size()*ms.size()*(locs)*i+xs_l.size()*ms.size()*(locs)*3*(t-t_il)+m_i]=D2[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i];
      resF0l[x_i*ms.size()+xs_l.size()*ms.size()*l_i+xs_l.size()*ms.size()*(locs)*i+xs_l.size()*ms.size()*(locs)*3*(t-t_il)+m_i]=F0[x_i*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*l_i+(xs_l.size()-1)*(ms.size()-1)*(locs)*i+m_i];
      }
     }
    } //close m loop
   }  //close x loop
  for (int xx=0; xx<(xs_l.size()-1); ++xx){
   for (int mm=0; mm<(ms.size()-1); ++mm){
    for (int ll=0; ll<locs; ++ll){
     for (int ii=0; ii<3; ++ii){
      F1s[xx*ms.size()+xs_l.size()*ms.size()*ll+xs_l.size()*ms.size()*(locs+1)*ii+mm]=F0[xx*(ms.size()-1)+(xs_l.size()-1)*(ms.size()-1)*ll+(xs_l.size()-1)*(ms.size()-1)*(locs)*ii+mm];
      }
     }
    }
   }
  }


//LINKING STEP: t=243 (day before birth of the calf) - sequential coupling BUT 242 in C++
 if(t==242){
  Rcpp::NumericVector F0((xs_p.size()-1)*(locs)*3); //array F0 to store max fitness at t
  Rcpp::NumericVector D((xs_p.size()-1)*(locs)*3);  //array D to store best strategy to choose

  //set fitness value to 0 when x<x_abo (if blubber mass is below x_abo, fetus is aborted):
  for(int ll = 0; ll < (locs); ++ll) {
   for(int ii = 0; ii < 3; ++ii) {
    F0[0*(locs)+(xs_p.size()-1)*(locs)*ii+ll]=0;
    D[0*(locs)+(xs_p.size()-1)*(locs)*ii+ll]=0;
    resDp[0*locs+xs_p.size()*locs*ii+xs_p.size()*(locs)*3*t+ll]=D[0*(locs)+(xs_p.size()-1)*(locs)*ii+ll];
    resDp2[0*locs+xs_p.size()*locs*ii+xs_p.size()*(locs)*3*t+ll]=D[0*(locs)+(xs_p.size()-1)*(locs)*ii+ll];
    resF0p[0*locs+xs_p.size()*locs*ii+xs_p.size()*(locs)*3*t+ll]=F0[0*(locs)+(xs_p.size()-1)*(locs)*ii+ll];
    }
   }

  for (int x_i=1; x_i<(xs_p.size()-1); ++x_i){                        //not below x_abo (i.e. xs_p[1]) because the foetus is aborted anyway at that blubber level, so it doesnt matter if the female lives or dies
   for (int l_i=0; l_i<2; ++l_i){                                     //she is in the breeding ground to give birth
    resDp[x_i*locs+xs_p.size()*locs*1+xs_p.size()*(locs)*3*t+l_i]=0;
    resDp2[x_i*locs+xs_p.size()*locs*1+xs_p.size()*(locs)*3*t+l_i]=0;
    resF0p[x_i*locs+xs_p.size()*locs*1+xs_p.size()*(locs)*3*t+l_i]=NAN;
    Rcpp::NumericVector patches(2);
    patches[0]=0;
    patches[1]=2;
    for (int in=0; in < patches.size(); ++in){                        //cycle over other two patch types
     int i = patches[in];
     Rcpp::NumericVector x1;
     Rcpp::NumericVector yju(30);
     Rcpp::NumericVector pju(30);
     for (int yjj=0; yjj<30; ++yjj){
      yju[yjj]=yj[2*30 + 3*30*l_i + 3*30*(locs+1)*t + yjj];  //yj[3,] (2 in C++) used because we are assuming female doesnt feed in this time step (which means that patch corresponds to a patch of type 3, without food)
      pju[yjj]=pj[2*30 + 3*30*l_i + 3*30*(locs+1)*t + yjj];
      }
     float for_hoursu;
     for_hoursu=for_hours[l_i*(T+1)+t];

     x1 = xs_p[x_i]+(- cr(xs_p[x_i]+Wc[t+timeS], lean, yju, w, beta, psi, tc, l_i, t, for_hoursu, scenarion[38])- ag(t+timeS,Wc_d,cost_kg,propg,Chg_tot))/u;
     Rcpp::NumericVector m1(x1.size());
     Rcpp::NumericVector F11(x1.size());
     for(int xx1=0; xx1<x1.size(); ++xx1){
      if(x1[xx1]<(x_abo-int_x)) x1[xx1]=(x_abo-int_x);   //truncated so that min is (x_abo-int_x)
      if(x1[xx1]>C) x1[xx1]=C;                           //truncated so that max is C
      m1[xx1]=mc+0.01;
      }
     F11=interp_mx(x1,m1,xc,mc,xs_l,ms,int_x,int_m,l_i,i,F1s,locs);                 //calculates corresponding fitness

     float V;
     if(sum(pju*x1)>=x_abo){
      V=sum(pju*F11);
      }
      else
      {
      V=0;
      }
     F0[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=V;
     D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=0;  //no behavioural choice
     resDp[x_i*locs+xs_p.size()*locs*i+xs_p.size()*(locs)*3*t+l_i]=D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i];
     resDp2[x_i*locs+xs_p.size()*locs*i+xs_p.size()*(locs)*3*t+l_i]=D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i];
     resF0p[x_i*locs+xs_p.size()*locs*i+xs_p.size()*(locs)*3*t+l_i]=F0[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i];
     }
    }
   }

   for (int xx=0; xx<(xs_p.size()-1); ++xx){
    for (int ll=0; ll<locs; ++ll){
     for (int ii=0; ii<3; ++ii){
      F1p[xx*(locs+1)+xs_p.size()*(locs+1)*ii+ll]=F0[xx*(locs)+(xs_p.size()-1)*(locs)*ii+ll];
      }
     }
    }

   for(int xx = 0; xx < xs_p.size(); ++xx) {
    for(int ll = 2; ll < (locs); ++ll) {      //from location 3 onwards
     for(int ii = 0; ii < 3; ++ii) {
      resF0p[xx*locs+xs_p.size()*locs*ii+xs_p.size()*(locs)*3*t+ll]=0;
      }
     }
    }
  }


//pregnancy
 if(t<242){
  Rcpp::NumericVector F0((xs_p.size()-1)*(locs)*3); //array F0 to store max fitness at t for different values of x, l and i
  Rcpp::NumericVector D((xs_p.size()-1)*(locs)*3);  //array D to store best strategy to choose depending on x, l and i
  Rcpp::NumericVector D2((xs_p.size()-1)*(locs)*3); //array D2 to store second best strategy to choose depending on x, l and i

  //set fitness value to 0 when x<x_abo (i.e. x_i=1; =0 in C++) (if blubber mass is below x_abo, fetus is aborted):
  for(int ll = 0; ll < (locs); ++ll) {
   for(int ii = 0; ii < 3; ++ii) {
    F0[0*(locs)+(xs_p.size()-1)*(locs)*ii+ll]=0;
    D[0*(locs)+(xs_p.size()-1)*(locs)*ii+ll]=0;
    resDp[0*locs+xs_p.size()*locs*ii+xs_p.size()*(locs)*3*t+ll]=D[0*(locs)+(xs_p.size()-1)*(locs)*ii+ll];
    resDp2[0*locs+xs_p.size()*locs*ii+xs_p.size()*(locs)*3*t+ll]=D[0*(locs)+(xs_p.size()-1)*(locs)*ii+ll];
    resF0p[0*locs+xs_p.size()*locs*ii+xs_p.size()*(locs)*3*t+ll]=F0[0*(locs)+(xs_p.size()-1)*(locs)*ii+ll];
    }
   }

  for (int x_i=1; x_i<(xs_p.size()-1); ++x_i){

    int l_i=0;
    resDp[x_i*locs+xs_p.size()*locs*1+xs_p.size()*(locs)*3*t+l_i]=0;
    resDp2[x_i*locs+xs_p.size()*locs*1+xs_p.size()*(locs)*3*t+l_i]=0;
    resF0p[x_i*locs+xs_p.size()*locs*1+xs_p.size()*(locs)*3*t+l_i]=NAN;
    Rcpp::NumericVector patches(2);
    patches[0]=0;
    patches[1]=2;
    for (int in=0; in < patches.size(); ++in){
     int i = patches[in];
     Rcpp::NumericVector x1;
     float x2;
     Rcpp::NumericVector yju(30);
     Rcpp::NumericVector pju(30);
     for (int yjj=0; yjj<30; ++yjj){
      yju[yjj]=yj[i*30 + 3*30*l_i + 3*30*(locs+1)*t + yjj];
      pju[yjj]=pj[i*30 + 3*30*l_i + 3*30*(locs+1)*t + yjj];
      }
     float for_hoursu;
     for_hoursu=for_hours[l_i*(T+1)+t];

     x1 = xs_p[x_i]+(- cf(xs_p[x_i]+Wc[t+timeS], lean, raamr, yju, w, beta, psi, tc, l_i, t, for_hoursu) - cr(xs_p[x_i]+Wc[t+timeS], lean, yju, w, beta, psi, tc, l_i, t, for_hoursu, scenarion[38])+ food(yju, w, beta, psi, tc, l_i, t, for_hoursu, rho, A)- ag(t+timeS,Wc_d,cost_kg,propg,Chg_tot))/u;
     Rcpp::NumericVector F11(x1.size());
     for(int xx1=0; xx1<x1.size(); ++xx1){
      if(x1[xx1]<(x_abo-int_x)) x1[xx1]=(x_abo-int_x);   //truncated so that min is (x_abo-int_x)
      if(x1[xx1]>C) x1[xx1]=C;                           //truncated so that max is C
      }
     F11=interp_x(x1,x_abo,xs_p,int_x,l_i,i,F1p,locs);   //calculates corresponding fitness

     x2 = xs_p[x_i] + (- cot(xs_p[x_i]+Wc[t+timeS],lean, scenarion[39])*d*(xs_p[x_i]+lean+Wc[t+timeS])- ag(t+timeS,Wc_d,cost_kg,propg,Chg_tot))/u;
     if(x2<(x_abo-int_x)) x2=(x_abo-int_x);              //truncated so that min is (x_abo-int_x)
     if(x2>C) x2=C;                                      //truncated so that max is C
     Rcpp::NumericVector F12(3);
     Rcpp::NumericVector x2v(1);
     x2v[0]=x2;
     for (int iii=0;iii<3;++iii){
      Rcpp::NumericVector tmp2=interp_x(x2v,x_abo,xs_p,int_x,l_i+1,iii,F1p,locs);
      F12[iii]=tmp2[0];
      }
     Rcpp::NumericVector lambda_f2(3);
     for (int la=0; la<3; ++la){
      lambda_f2[la]=lambda[(l_i+1)*3 + (locs+1)*3*(t+1) + la];                 //lambda depends on location and time
      }

     Rcpp::NumericVector F13(3);
     for (int iii=0;iii<3;++iii){
      Rcpp::NumericVector tmp3=interp_x(x2v,x_abo,xs_p,int_x,l_i,iii,F1p,locs);
      F13[iii]=tmp3[0];
      }
     Rcpp::NumericVector lambda_f3(3);
     for (int la=0; la<3; ++la){
      lambda_f3[la]=lambda[(l_i)*3 + (locs+1)*3*(t+1) + la];                   //lambda depends on location and time
      }


     Rcpp::NumericVector V(3);
     V[0]=sum(pju*F11);
     V[1]=sum(lambda_f2*F12);
     V[2]=sum(lambda_f3*F13);
     F0[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=max(V);
     V[0]=roundf(V[0]*pow(10,14))/pow(10,14);
     V[1]=roundf(V[1]*pow(10,14))/pow(10,14);
     V[2]=roundf(V[2]*pow(10,14))/pow(10,14);

     if ((V[0]==V[1])&&(V[1]==V[2])){
       D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=4;
       }
       else
       {
       if((V[0]==V[1])&&(V[1]>V[2])){
        D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=5;
        }
        else
        {
        if((V[0]==V[2])&&(V[2]>V[1])){
         D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=6;
         }
         else
         {
         if((V[1]==V[2])&&(V[2]>V[0])){
          D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=7;
          }
          else
          {
          D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=(vecmax(V)+1);
          }
         }
        }
       }
     D2[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=vec2max(V);

     resF0p[x_i*locs+xs_p.size()*locs*i+xs_p.size()*(locs)*3*t+l_i]=F0[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i];
     resDp[x_i*locs+xs_p.size()*locs*i+xs_p.size()*(locs)*3*t+l_i]=D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i];
     resDp2[x_i*locs+xs_p.size()*locs*i+xs_p.size()*(locs)*3*t+l_i]=D2[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i];
     }

    l_i=1;
    resDp[x_i*locs+xs_p.size()*locs*1+xs_p.size()*(locs)*3*t+l_i]=0;
    resDp2[x_i*locs+xs_p.size()*locs*1+xs_p.size()*(locs)*3*t+l_i]=0;
    resF0p[x_i*locs+xs_p.size()*locs*1+xs_p.size()*(locs)*3*t+l_i]=NAN;
    patches[0]=0;
    patches[1]=2;
    for (int in=0; in < patches.size(); ++in){                               //cycle over other two patch types
     int i = patches[in];
     Rcpp::NumericVector x1;
     float x2;
     Rcpp::NumericVector yju(30);
     Rcpp::NumericVector pju(30);
     for (int yjj=0; yjj<30; ++yjj){
      yju[yjj]=yj[i*30 + 3*30*l_i + 3*30*(locs+1)*t + yjj];
      pju[yjj]=pj[i*30 + 3*30*l_i + 3*30*(locs+1)*t + yjj];
      }
     float for_hoursu;
     for_hoursu=for_hours[l_i*(T+1)+t];

     x1 = xs_p[x_i]+(- cf(xs_p[x_i]+Wc[t+timeS], lean, raamr, yju, w, beta, psi, tc, l_i, t, for_hoursu) - cr(xs_p[x_i]+Wc[t+timeS], lean, yju, w, beta, psi, tc, l_i, t, for_hoursu, scenarion[38])+ food(yju, w, beta, psi, tc, l_i, t, for_hoursu, rho, A)- ag(t+timeS,Wc_d,cost_kg,propg,Chg_tot))/u;
     Rcpp::NumericVector F11(x1.size());
     for(int xx1=0; xx1<x1.size(); ++xx1){
      if(x1[xx1]<(x_abo-int_x)) x1[xx1]=(x_abo-int_x);   //truncated so that min is (x_abo-int_x)
      if(x1[xx1]>C) x1[xx1]=C;                           //truncated so that max is C
      }
     F11=interp_x(x1,x_abo,xs_p,int_x,l_i,i,F1p,locs);   //calculates corresponding fitness

     x2 = xs_p[x_i] + (- cot(xs_p[x_i]+Wc[t+timeS],lean, scenarion[39])*d*(xs_p[x_i]+lean+Wc[t+timeS])- ag(t+timeS,Wc_d,cost_kg,propg,Chg_tot))/u;
     if(x2<(x_abo-int_x)) x2=(x_abo-int_x);              //truncated so that min is (x_abo-int_x)
     if(x2>C) x2=C;                                      //truncated so that max is C
     Rcpp::NumericVector F12(3);
     Rcpp::NumericVector x2v(1);
     x2v[0]=x2;
     for (int iii=0;iii<3;++iii){
      Rcpp::NumericVector tmp2=interp_x(x2v,x_abo,xs_p,int_x,l_i+1,iii,F1p,locs);
      F12[iii]=tmp2[0];
      }
     Rcpp::NumericVector lambda_f2(3);
     for (int la=0; la<3; ++la){
      lambda_f2[la]=lambda[(l_i+1)*3 + (locs+1)*3*(t+1) + la];                 //lambda depends on location and time
      }

     Rcpp::NumericVector F13(3);
     for (int iii=0;iii<3;++iii){
      Rcpp::NumericVector tmp3=interp_x(x2v,x_abo,xs_p,int_x,l_i-1,iii,F1p,locs);
      F13[iii]=tmp3[0];
      }
     Rcpp::NumericVector lambda_f3(3);
     for (int la=0; la<3; ++la){
      lambda_f3[la]=lambda[(l_i-1)*3 + (locs+1)*3*(t+1) + la];                 //lambda depends on location and time
      }

     Rcpp::NumericVector F14(3);
     for (int iii=0;iii<3;++iii){
      Rcpp::NumericVector tmp4=interp_x(x2v,x_abo,xs_p,int_x,l_i,iii,F1p,locs);
      F14[iii]=tmp4[0];
      }
     Rcpp::NumericVector lambda_f4(3);
     for (int la=0; la<3; ++la){
      lambda_f4[la]=lambda[(l_i)*3 + (locs+1)*3*(t+1) + la];                   //lambda depends on location and time
      }

     Rcpp::NumericVector V2(4);
     V2[0]=sum(pju*F11);
     V2[1]=sum(lambda_f2*F12);
     V2[2]=sum(lambda_f3*F13);
     V2[3]=sum(lambda_f4*F14);
     F0[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=max(V2);
     V2[0]=roundf(V2[0]*pow(10,14))/pow(10,14);
     V2[1]=roundf(V2[1]*pow(10,14))/pow(10,14);
     V2[2]=roundf(V2[2]*pow(10,14))/pow(10,14);
     V2[3]=roundf(V2[3]*pow(10,14))/pow(10,14);
     if ((V2[0]==V2[1])&&(V2[1]==V2[2])&&(V2[2]==V2[3])){
       D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=5;
       }
       else
       {
       if((V2[0]==V2[1])&&(V2[1]==V2[2])&&(V2[2]>V2[3])){
        D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=6;
        }
        else
        {
        if((V2[0]==V2[1])&&(V2[1]==V2[3])&&(V2[3]>V2[2])){
         D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=7;
         }
         else
         {
         if((V2[0]==V2[2])&&(V2[2]==V2[3])&&(V2[3]>V2[1])){
          D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=8;
          }
          else
          {
          if((V2[1]==V2[2])&&(V2[2]==V2[3])&&(V2[3]>V2[0])){
           D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=9;
           }
           else
           {
           if((V2[0]==V2[2])&&(V2[2]==max(V2))&&(V2[2]>V2[3])){
            D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=10;
            }
            else
            {
            if((V2[0]==V2[1])&&(V2[1]==max(V2))&&(V2[1]>V2[3])){
             D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=11;
             }
             else
             {
             if((V2[0]==V2[3])&&(V2[3]==max(V2))&&(V2[3]>V2[1])){
              D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=12;
              }
              else
              {
              if((V2[1]==V2[2])&&(V2[2]==max(V2))&&(V2[2]>V2[0])){
               D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=13;
               }
               else
               {
               if((V2[1]==V2[3])&&(V2[3]==max(V2))&&(V2[3]>V2[0])){
                D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=14;
                }
                else
                {
                if((V2[2]==V2[3])&&(V2[3]==max(V2))&&(V2[3]>V2[0])){
                 D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=15;
                 }
                 else
                 {
                 D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=(vecmax(V2)+1);
                 }
                }
               }
              }
             }
            }
           }
          }
         }
        }
       }
     D2[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=vec2max(V2);

     resF0p[x_i*locs+xs_p.size()*locs*i+xs_p.size()*(locs)*3*t+l_i]=F0[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i];
     resDp[x_i*locs+xs_p.size()*locs*i+xs_p.size()*(locs)*3*t+l_i]=D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i];
     resDp2[x_i*locs+xs_p.size()*locs*i+xs_p.size()*(locs)*3*t+l_i]=D2[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i];
     }

    for (int l_i=2;l_i<locs; ++l_i){                           //cycle over all other locations
     for (int i=0; i<3; ++i){                                  //cycle over all patch types
      Rcpp::NumericVector x1;
      float x2;
      Rcpp::NumericVector yju(30);
      Rcpp::NumericVector pju(30);
      for (int yjj=0; yjj<30; ++yjj){
       yju[yjj]=yj[i*30 + 3*30*l_i + 3*30*(locs+1)*t + yjj];
       pju[yjj]=pj[i*30 + 3*30*l_i + 3*30*(locs+1)*t + yjj];
       }
      float for_hoursu;
      for_hoursu=for_hours[l_i*(T+1)+t];

      x1 = xs_p[x_i]+(- cf(xs_p[x_i]+Wc[t+timeS], lean, raamr, yju, w, beta, psi, tc, l_i, t, for_hoursu) - cr(xs_p[x_i]+Wc[t+timeS], lean, yju, w, beta, psi, tc, l_i, t, for_hoursu, scenarion[38])+ food(yju, w, beta, psi, tc, l_i, t, for_hoursu, rho, A)- ag(t+timeS,Wc_d,cost_kg,propg,Chg_tot))/u;
      Rcpp::NumericVector F11(x1.size());
      for(int xx1=0; xx1<x1.size(); ++xx1){
       if(x1[xx1]<(x_abo-int_x)) x1[xx1]=(x_abo-int_x);   //truncated so that min is (x_abo-int_x)
       if(x1[xx1]>C) x1[xx1]=C;                           //truncated so that max is C
       }
      F11=interp_x(x1,x_abo,xs_p,int_x,l_i,i,F1p,locs);   //calculates corresponding fitness

      x2 = xs_p[x_i] + (- cot(xs_p[x_i]+Wc[t+timeS],lean, scenarion[39])*d*(xs_p[x_i]+lean+Wc[t+timeS])- ag(t+timeS,Wc_d,cost_kg,propg,Chg_tot))/u;
      if(x2<(x_abo-int_x)) x2=(x_abo-int_x);              //truncated so that min is (x_abo-int_x)
      if(x2>C) x2=C;                                      //truncated so that max is C
      Rcpp::NumericVector F12(3);
      Rcpp::NumericVector x2v(1);
      x2v[0]=x2;
      for (int iii=0;iii<3;++iii){
       Rcpp::NumericVector tmp2=interp_x(x2v,x_abo,xs_p,int_x,l_i+1,iii,F1p,locs);
       F12[iii]=tmp2[0];
       }
      Rcpp::NumericVector lambda_f2(3);
      for (int la=0; la<3; ++la){
       lambda_f2[la]=lambda[(l_i+1)*3 + (locs+1)*3*(t+1) + la];                 //lambda depends on location and time
       }

      Rcpp::NumericVector F13(3);
      for (int iii=0;iii<3;++iii){
       Rcpp::NumericVector tmp3=interp_x(x2v,x_abo,xs_p,int_x,l_i-1,iii,F1p,locs);
       F13[iii]=tmp3[0];
       }
      Rcpp::NumericVector lambda_f3(3);
      for (int la=0; la<3; ++la){
       lambda_f3[la]=lambda[(l_i-1)*3 + (locs+1)*3*(t+1) + la];                 //lambda depends on location and time
       }

     Rcpp::NumericVector V3(3);
     V3[0]=sum(pju*F11);
     V3[1]=sum(lambda_f2*F12);
     V3[2]=sum(lambda_f3*F13);
     F0[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=max(V3);
     V3[0]=roundf(V3[0]*pow(10,14))/pow(10,14);
     V3[1]=roundf(V3[1]*pow(10,14))/pow(10,14);
     V3[2]=roundf(V3[2]*pow(10,14))/pow(10,14);
     if ((V3[0]==V3[1])&&(V3[1]==V3[2])){
       D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=4;
       }
       else
       {
       if((V3[0]==V3[1])&&(V3[1]>V3[2])){
        D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=5;
        }
        else
        {
        if((V3[0]==V3[2])&&(V3[2]>V3[1])){
         D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=6;
         }
         else
         {
         if((V3[1]==V3[2])&&(V3[2]>V3[0])){
          D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=7;
          }
          else
          {
          D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=(vecmax(V3)+1);
          }
         }
        }
       }
     D2[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i]=vec2max(V3);

     resF0p[x_i*locs+xs_p.size()*locs*i+xs_p.size()*(locs)*3*t+l_i]=F0[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i];
     resDp[x_i*locs+xs_p.size()*locs*i+xs_p.size()*(locs)*3*t+l_i]=D[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i];
     resDp2[x_i*locs+xs_p.size()*locs*i+xs_p.size()*(locs)*3*t+l_i]=D2[x_i*(locs)+(xs_p.size()-1)*(locs)*i+l_i];
     } //close i
    }  //close l

   } //close x

  for (int xx=0; xx<(xs_p.size()-1); ++xx){
   for (int ll=0; ll<locs; ++ll){
    for (int ii=0; ii<3; ++ii){
     F1p[xx*(locs+1)+xs_p.size()*(locs+1)*ii+ll]=F0[xx*(locs)+(xs_p.size()-1)*(locs)*ii+ll];
     }
    }
   }
  }

}//end of temporal loop

 Rcpp::List ret;
 ret["resDp"] = resDp;
 ret["resF0p"] = resF0p;
 ret["resDl"] = resDl;
 ret["resF0l"] = resF0l;
 ret["resDp2"] = resDp2;
 ret["resDl2"] = resDl2;
 return ret;


}')






#######################################################Forward iteration function###########################################################################################################################################################################################################################################################################################################################################################
sourceCpp(code='

#include <math.h>
using namespace std;
// [[Rcpp::depends(RcppArmadillo)]]
#include <RcppArmadillo.h>
#include <RcppArmadilloExtensions/sample.h>

int vecmax(Rcpp::NumericVector x) {
  Rcpp::NumericVector::iterator it = std::max_element(x.begin(), x.end());
  return it - x.begin();
}

float roundTo2dec(float x)
 {
       Rcpp::NumericVector vec(1);
       vec[0]=x;
       vec=Rcpp::round(vec,2);
       return vec[0];
 }

float roundToThousandths(float x)
 {
     x /=1000;
       return floor(x + 0.5) * 1000;
 }

float roundToHundreds(float x)
 {
     x /=100;
       return floor(x + 0.5) * 100;
 }

Rcpp::NumericVector sequence(float minx, float maxx, float intx)
 {
     int n=(maxx-minx)/intx;
     Rcpp::NumericVector x(n+1);
     double total=minx;
     x[0]=total;
     for(int i = 1; i < (n+1); ++i) {
       total = total+intx;
       x[i] = total;
       }
     return x;
 }


float ag(int gtime, Rcpp::NumericVector Wc_d, float cost_kg, Rcpp::NumericVector propg, float Chg_tot)                                   //function to calculate gestational costs given stage of pregnancy in days from start
 {
 return Wc_d[gtime]*cost_kg + propg[gtime]*Chg_tot;
 }


float cot(float blubber, float lean, float scen)
 {
 float cost=7.79*pow(blubber + lean,-0.29)*scen;
 return cost;
 }

float blub_perc(float milk, int t,float blub_per, float me, float Er, float u, int t_il, Rcpp::NumericVector leand)
 {
           int day=t-t_il;
           float bm=leand[day]*blub_per/(1-blub_per);     //current blubber mass, derived from % of blubber and lean mass in this time step
           float Ed=milk*me;                              //energy received by the calf given amount of milk
           float S=Ed-Er;                                 //balance of energy usable to deposit blubber (or to cover costs of survival and growth) - it can be negative
           float D=S/u;                                   //kg of blubber that are deposited/used as the result of energy balance
           float nbm=D+bm;                                //new blubber mass
           float perc=nbm/(leand[day+1]+nbm);
           return perc;             //return new blubber %
 }


float extra_m(float nper, int t, int t_il, float mC, float E, Rcpp::NumericVector leand)                                 //calculates how much blubber mass the calf would receive above the maximum % blubber (so that it is summed back to mothers mass)
 {
           float pMax=leand[t-t_il+1]*mC/(1-mC);      //this is the maximum blubber mass the calf can have on the following day
           float extra;
           float nbm=leand[t-t_il+1]*nper/(1-nper);   //this is the blubber mass it would achieve based on the milk the mother can give
           if (nper>mC){
            extra=(nbm-pMax);
            }else{
            extra=0;
            }
           extra=extra/E;
           return extra; //returns extra mass (which is not delivered to the calf but retained by the female (with loss in energy transfer)
 }


Rcpp::NumericVector lr(Rcpp::NumericVector dens, Rcpp::NumericVector w)      //function to extract lunging rate given krill density
 {
    Rcpp::NumericVector wa(dens.size());
    Rcpp::LogicalVector lll=(dens<0.2);
    Rcpp::LogicalVector mmm=(dens>=0.2 & dens<=0.4);
    Rcpp::LogicalVector hhh=(dens>0.4);

    for (int i=0; i<dens.size(); i++){
     if(lll[i]) wa[i]=w[0];
     if(mmm[i]) wa[i]=w[1];
     if(hhh[i]) wa[i]=w[2];
     }
    return wa;
 }


Rcpp::NumericVector tf2(Rcpp::NumericVector dens, Rcpp::NumericVector w, float beta, float psi, int tc, int l_i, int t, float for_hoursu, int disturbance, float severity)      //function to calculate time spent feeding (given patch density and anatomical limitations at given length)
 {
    Rcpp::NumericVector ti(dens.size());
    Rcpp::NumericVector ni(dens.size());
    Rcpp::NumericVector lunge=lr(dens,w);
    Rcpp::NumericVector hours(dens.size());
    Rcpp::NumericVector frate(dens.size());

    for (int i=0; i<dens.size(); i++){
     frate[i]=lunge[i]*beta*dens[i];           //max amount of food per hour
     if(frate[i]!=0) ti[i]=psi/frate[i];       //time (hours) to fill forestomach (depends on density; if density is 0, time is 0)
     float ntime;
     if(disturbance==1){
      ntime=for_hoursu*severity;
      }else{
      ntime=for_hoursu;
      }
     if((ti[i]>tc) && (ti[i]!=0)) ni[i]=ntime/ti[i];   //number of times it can fill the forestomach in a 12 hour period
     if((ti[i]<=tc) && (ti[i]!=0)) ni[i]=ntime/tc;
     hours[i]=ti[i]*ni[i];
     }
    return hours;
 }

// [[Rcpp::export]]

Rcpp::NumericVector cf2(float blubber, float lean, float raamr, Rcpp::NumericVector dens, Rcpp::NumericVector w, float beta, float psi, int tc, int l_i, int t, float for_hoursu, int disturbance, float severity) //function to calculate cost of feeding
 {
    Rcpp::NumericVector cfeed(dens.size());
    Rcpp::NumericVector hours=tf2(dens,w,beta,psi,tc,l_i,t,for_hoursu,disturbance,severity);
    for (int i=0; i<dens.size(); i++){
     cfeed[i]=hours[i]*60*60*raamr*(blubber+lean)/1000; //multiply RAAMR/Mc(W) by seconds and by kg, and divide by 1000 to get kJ
     }
    return cfeed;
 }


Rcpp::NumericVector cr2(float blubber, float lean, Rcpp::NumericVector dens, Rcpp::NumericVector w, float beta, float psi, int tc, int l_i, int t, float for_hoursu, int disturbance, float severity, float scen)         //function to calculate cost of resting (given patch density to calculate time spent foraging, weight and anatomical limitations at given length)
 {
    float BMR=4*(pow(blubber + lean,0.75))*(24*60*60)/1000;  //kJ/d  (from Kleiber equation BMR(W)=4Mc^0.75: multiply by seconds and by kg and divide by 1000 to get kJ
    float BMRr=2*BMR*scen;                                        //MR while resting kJ/d (as in Potvin et al 2012)
    Rcpp::NumericVector crest(dens.size());
    Rcpp::NumericVector hours=tf2(dens,w,beta,psi,tc,l_i,t,for_hoursu,disturbance,severity);
    for (int i=0; i<dens.size(); i++){
     crest[i]=(24-hours[i])/24*BMRr;
     }
    return crest;
 }


Rcpp::NumericVector food2(Rcpp::NumericVector dens, Rcpp::NumericVector w, float beta, float psi, int tc, int l_i, int t, float for_hoursu, int rho, float A, int disturbance, float severity)               //function to calculate amount of energy from engulfed food (given patch density and anatomical limitations at given length)
 {
    Rcpp::NumericVector ti(dens.size());
    Rcpp::NumericVector ni(dens.size());
    Rcpp::NumericVector lunge=lr(dens,w);
    Rcpp::NumericVector fq(dens.size());
    Rcpp::NumericVector frate(dens.size());

    for (int i=0; i<dens.size(); i++){
     frate[i]=lunge[i]*beta*dens[i];           //max amount of food per hour
     if(frate[i]!=0) ti[i]=psi/frate[i];       //time (hours) to fill forestomach (depends on density; if density is 0, time is 0)
     float ntime;
     if(disturbance==1){
      ntime=for_hoursu*severity;       //severity is the reduction in amount of time available to feed
      }else{
      ntime=for_hoursu;
      }
     if((ti[i]>tc) && (ti[i]!=0)) ni[i]=ntime/ti[i];   //number of times it can fill the forestomach in a 12 hour period
     if((ti[i]<=tc) && (ti[i]!=0)) ni[i]=ntime/tc;
     fq[i]=ni[i]*psi*rho*A;
     }
    return fq;
 }


//functions for interpolations

Rcpp::NumericVector interp_mx(Rcpp::NumericVector x1, Rcpp::NumericVector m1, float xc, float mc, Rcpp::NumericVector xs_l, Rcpp::NumericVector ms, int int_x, float int_m, int l_i, int i,Rcpp::NumericVector F1s,int locs)  //interpolation for lactating phase
 {
      Rcpp::NumericVector delta_x(x1.size());
      Rcpp::NumericVector delta_m(m1.size());
      Rcpp::NumericVector interpolm(x1.size());
      for(int j=0; j<x1.size(); j++){
         int x1t_i=(x1[j]-xc)/int_x;
         float xi=(xs_l.size()-1)/(max(xs_l)-xc)*(x1[j]-xc);
         delta_x[j]=xi-x1t_i;

         int m1t_i=(m1[j]-mc)/int_m;
         float mi=(ms.size()-1)/(max(ms)-mc)*(m1[j]-mc);
         delta_m[j]=mi-m1t_i;

         interpolm[j]=(1-delta_x[j])*(1-delta_m[j])*F1s[x1t_i*ms.size()+xs_l.size()*ms.size()*l_i+xs_l.size()*ms.size()*(locs+1)*i+m1t_i] +
         (1-delta_x[j])*delta_m[j]*F1s[x1t_i*ms.size()+xs_l.size()*ms.size()*l_i+xs_l.size()*ms.size()*(locs+1)*i+m1t_i+1]+
         delta_x[j]*(1-delta_m[j])*F1s[(x1t_i+1)*ms.size()+xs_l.size()*ms.size()*l_i+xs_l.size()*ms.size()*(locs+1)*i+m1t_i] +
         delta_x[j]*delta_m[j]*F1s[(x1t_i+1)*ms.size()+xs_l.size()*ms.size()*l_i+xs_l.size()*ms.size()*(locs+1)*i+m1t_i+1];
         }
      return interpolm;
 }



Rcpp::NumericVector interp_x(Rcpp::NumericVector x1, float x_abo, Rcpp::NumericVector xs_p, int int_x, int l_i,int i, Rcpp::NumericVector F1p,int locs)          //interpolation for pregnant phase
 {
      Rcpp::NumericVector delta_x(x1.size());
      Rcpp::NumericVector interpolm(x1.size());
      for(int j=0; j<x1.size(); j++){
         int x1t_i=(x1[j]-(x_abo-int_x))/int_x;
         float xi=(xs_p.size()-1)/(max(xs_p)-(x_abo-int_x))*(x1[j]-(x_abo-int_x));
         delta_x[j]=xi-x1t_i;

         interpolm[j]=(1-delta_x[j])*F1p[x1t_i*(locs+1)+(xs_p.size())*(locs+1)*i+l_i] +
         delta_x[j]*F1p[(x1t_i+1)*(locs+1)+(xs_p.size())*(locs+1)*i+l_i];
         }
      return interpolm;
 }

int oneMultinom(Rcpp::NumericVector probs)
 {
    int k = probs.size();
    Rcpp::IntegerVector ans(k);
    rmultinom(1, probs.begin(), k, ans.begin());
    for(int p=0; p<ans.size(); ++p){
     if(ans[p]!=0) return p;
     }
 }

// [[Rcpp::plugins(cpp11)]]
// [[Rcpp::export]]

Rcpp::NumericVector vecrbinom(double size, Rcpp::NumericVector prob)
 {
    int n=prob.size();
    Rcpp::NumericVector v = Rcpp::no_init(n);
    std::transform( prob.begin(), prob.end(), v.begin(), [=](double p){ return R::rbinom(size, p); });
    return(v);
    }

#include <RcppArmadilloExtensions/sample.h>
// [[Rcpp::depends(RcppArmadillo)]]

using namespace Rcpp;

// [[Rcpp::export]]

Rcpp::List forwardIT_d_2(int ni,Rcpp::NumericVector resDp, Rcpp::NumericVector resDl,Rcpp::NumericVector resDp2, Rcpp::NumericVector resDl2, Rcpp::NumericVector yj, Rcpp::NumericVector pj, Rcpp::NumericVector for_hours, Rcpp::NumericVector lambda, float severity, Rcpp::NumericVector dist_locs, Rcpp::NumericVector responsive, Rcpp::NumericVector scenarion)    //main function for forward iteration
{

int T, locs, feed_g, length, int_x, d, back, timeS, Mmax, tc, rho, gamma;
float weight, lean, max_perc, min_perc, death_perc, C, x_abo, x_lac, xc, mc, mC, int_m, cost_kg, Chg_tot, me, E, wb, lb, ww, lw, lgd, meat_cal, MRc, LEd, u, raamr, beta, psi, A, m50;
Rcpp::NumericVector xs_l, xs_p, gtime, ms, Wc, Wc_d(334), propg(334), Chg(334), leand(212), w(3);

T=454;     //max time steps (1 April - end of June of the following year; weaning after 7 months, i.e. 210 days)
locs=36;   //number of locations
feed_g=15; //this is the location at which the main summer feeding ground starts (California current)
int t_il=243;

//morphometrics
length=22;                                      //from Gilpatrick and Perryman 2008, whales were on average smaller than in the Southern Ocean (22 m vs 25 m in the SO)
weight=4.6*pow(length,3.05)*scenarion[0];       //average weight of a whale of that length; length(m)-weight(kg) relationship  from Lockyer 1976
lean=roundToHundreds(0.73*scenarion[1]*weight); //average mass of lean tissue (73%)

//extremes for x:
max_perc=0.35;                             //maximum observed % of blubber in the body (Lockyer data 1976 and 1981)
min_perc=0.16;                             //minimum observed % of blubber in the body (Lockyer data 1976 and 1981)
death_perc=0.05;                           //% of blubber at which the whale dies
C=roundToThousandths(max_perc*lean/(1-max_perc)*scenarion[2]);        //max blubber mass a female can reach
x_abo=roundToThousandths(min_perc*lean/(1-min_perc)*scenarion[4]);    //minimum blubber mass to carry on pregnancy. If blubber falls below this threshold: abortion
x_lac=roundToThousandths(min_perc*lean/(1-min_perc)*scenarion[5]);    //minimum blubber mass to deliver milk. If blubber falls below this threshold: no lactation
xc=roundToThousandths(death_perc*lean/(1-death_perc)*scenarion[3]);   //at this blubber mass, the female dies (automatic abortion if pregnant, but lactating calf can survive)
int_x=500;

xs_l=sequence(xc,C+int_x,int_x);           //possible values of blubber mass. Upper limit needs to be C+int for interpolation (lactation)
xs_p=sequence(x_abo-int_x,C+int_x,int_x);  //possible values of blubber mass. Upper limit needs to be C+int for interpolation (pregnancy)

//extremes for m (% calf blubber mass):
mc=roundTo2dec(0.05*scenarion[7]);     //min % of blubber mass (at 5% blubber mass, calf dies)
mC=roundTo2dec(0.44*scenarion[6]);     //max % of blubber mass (calf cannot have more than 44% fat)
int_m=0.01; //for calf blubber bins (1%)
ms=sequence(mc,roundTo2dec(0.44*scenarion[6]+int_m),int_m);

//////Energetics functions//////

//movement/migration
d = 100;                                                  //in km; distance they can cover in a day at optimal speed of 1.1 m/s (1.1*60*60*24/1000)
back = 2;                                                 //location after which female has the option to go backwards

//pregnancy (gestation costs)
gtime=sequence(1,334,1);                         //gestation is 11 months
Wc=pow(0.52*(gtime-73),3)/1000*scenarion[14];    //foetal growth for blue whales in kg, where t is the number of days since conception. From Hugget and Widdas 1951 (in Lockyer 1986)
Wc[Wc<0.001]=0;                                  //set weight of foetus to 0 when less than 1g
cost_kg=2940*4.184*scenarion[13];                //kJ/kg of foetus weight. 2940 kCal/kg from Lockyer (1987 and 2007) (1kCal=4.184 kJ)
for(int i = 0; i < (Wc.size()-1); ++i) {
 Wc_d[i]=(Wc[i+1]-Wc[i]);
 propg[i]=(Wc[i+1]-Wc[i])/max(Wc);
 }
propg[Wc.size()-1]=NAN;
Chg_tot=4400*4.184*pow(max(Wc),1.2)*scenarion[15]; //total cost of heat of gestation (from Lockyer 2007, transformed to kJ)
Chg=propg*Chg_tot;                                 //heat of gestation (in kJ/day) depending on day of pregnancy (i.e on proportional growth on that day)
timeS=90;                                          //at the beginning of the modelled period (1 Apr), the female is already 90 days pregnant

//lactation and calf growth
Mmax=220*scenarion[10];      //max milk delivered daily. I used the midpoint given by Oftedal 1997.
me=4137*4.184*scenarion[11]; //energy content (kJ/kg) of milk, 4137 kCal/kg (Lockyer 1986)
E=0.9*scenarion[12];         //90% mammary gland efficiency (Lockyer 2007)
wb=Wc[Wc.size()-1];          //weight at birth kg (from gestation growth curve)
lb=wb*0.94*scenarion[17];    //lean mass at birth (assuming calf is mostly lean at birth), but blubber % needs to be one interval above threshold for death
ww=17000*scenarion[18]+wb;   //avg estimated growth (Lockyer) + mass at birth (i.e. mass at weaning)
lw=ww*0.73*scenarion[19];    //lean mass at weaning we assume that the weaned calf has on average same % of blubber and lean mass as adult
lgd=(lw-lb)/211;             //calf avg daily growth in lean mass; kg/d

leand[0]=lb;
for(int i = 1; i < 212; ++i) {
 leand[i]= leand[i-1] + lgd; //total lean mass on each day, including the first day. calf gets weaned after the last day of lactation (i.e. on day post-birth 211)
 }
meat_cal=1500*4.184*scenarion[20]; //kJ/kg, from kcal/kg value by Petrides and Swank 1965 and Brody 1968 in Lockyer 1981

LEd=lgd*meat_cal;                  //kJ required to grow each day
MRc=200000*4.184*scenarion[21];    //kJ to sustain maintainance/metabolic rate Lockyer 1981(1kCal=4.184 kJ)

float Er=MRc + LEd;                //total daily amount of energy required by the calf to survive and grow
int daysize=285*scenarion[16];     //threshold time for the calf to be able to travel outside the breeding ground with its mother

//feeding and resting
u=31798*scenarion[22];             //Utilizable energy (kJ/kg) in blubber (Lockyer 1981)
w[0]=10*scenarion[23];             //number of lunges per hour (Goldbogen et al. 2011 and 2012, and Hazen et al. 2015 - 2, 4, or 6 lunges per dive on average (depending on krill density: <0.2, 0.2-0.4, >0.4); dive is 9.8 min + 2.7 min at the surface, i.e. 12.5 min on average, i.e. 5 dives/hour = 5*c(2,4,6)=c(10,20,30))
w[1]=20*scenarion[24];             //number of lunges per hour (Goldbogen et al. 2011 and 2012, and Hazen et al. 2015 - 2, 4, or 6 lunges per dive on average (depending on krill density: <0.2, 0.2-0.4, >0.4); dive is 9.8 min + 2.7 min at the surface, i.e. 12.5 min on average, i.e. 5 dives/hour = 5*c(2,4,6)=c(10,20,30))
w[2]=30*scenarion[25];             //number of lunges per hour (Goldbogen et al. 2011 and 2012, and Hazen et al. 2015 - 2, 4, or 6 lunges per dive on average (depending on krill density: <0.2, 0.2-0.4, >0.4); dive is 9.8 min + 2.7 min at the surface, i.e. 12.5 min on average, i.e. 5 dives/hour = 5*c(2,4,6)=c(10,20,30))
raamr= 26.23*pow(length,-0.809)*scenarion[26]; //in W/kg; this is RAAMR/Mc
beta=0.0011*pow(length,3.56)*scenarion[27];    //volume of water ingested in a lunge
tc=4*scenarion[28];                            //number of hours to clear forestomach
psi=0.47*pow(length,2.88)*scenarion[29];       //forestomach capacity - adjusted from fin whales as in Weidenmann et al 2011
rho=4184*scenarion[30];                        //kJ/kg amount of energy in krill (Lockyer 1981 in Weidenmann et al 2011); 4600 kj/kg Goldbogen 2011
A=0.84*scenarion[31];                          //assimilation efficiency (0.84 in Goldbogen et al 2011; Costa and Williams 2000: 0.8)

//terminal function - lactating
gamma=8*scenarion[9];  //steepness of fitness function
m50=0.27*scenarion[8]; //calf survival probability is 50% when its blubber % is the same as the observed one for adults

//initiate arrays

Rcpp::NumericVector x(ni*T);   //array for blubber mass
for (int k=0; k<(ni*T); ++k) x[k] = NAN;
Rcpp::NumericVector initsx=RcppArmadillo::sample(sequence(x_abo,C,1),ni,TRUE);
for (int k=0; k<ni; ++k) x[k*T+0]=initsx[k];
Rcpp::NumericVector i(ni*T);   //array for patch type
for (int k=0; k<(ni*T); ++k) i[k] = NAN;
Rcpp::NumericVector ipatches(2);
ipatches[0]=0;
ipatches[1]=2;
Rcpp::NumericVector initsi=RcppArmadillo::sample(ipatches,ni,TRUE);
for (int k=0; k<ni; ++k) i[k*T+0]=initsi[k];
Rcpp::NumericVector l(ni*T);   //array for location
for (int k=0; k<(ni*T); ++k) l[k] = NAN;
Rcpp::NumericVector ilocs(2);
ilocs[0]=0;
ilocs[1]=1;
Rcpp::NumericVector initsl=RcppArmadillo::sample(ilocs,ni,TRUE);
for (int k=0; k<ni; ++k) l[k*T+0]=initsl[k];
Rcpp::NumericVector st(ni*T);  //array for strategy
for (int k=0; k<(ni*T); ++k) st[k] = NAN;
Rcpp::NumericVector fetus_calf(ni*T);  //array to store whether the foetus/calf is still alive
for (int k=0; k<(ni*T); ++k) fetus_calf[k] = NAN;
for (int k=0; k<ni; ++k) fetus_calf[k*T+0]=1;
Rcpp::NumericVector m(ni*T);           //array to store % calf blubber mass
for (int k=0; k<ni; ++k) m[k*T+t_il]=mc+0.01;
Rcpp::NumericVector disturbed(ni*T);   //array to store disturbance
Rcpp::NumericVector resp(ni*T);        //array to store responses

Rcpp::NumericVector consec(ni);        //vector to store temporarily how many consecutive days spent in a patch
int degrade=21*scenarion[40];                        //number of days after which a patch degrades in quality

for (int k=0; k<ni; ++k) {             //k is the individual
 for (int t=0; t<(T-1); ++t){
  //pregnancy
  if(t<242){
   if(x[k*T+t]<x_abo){
    x[k*T+t+1]=45000;                                 //if blubber is below x_abo, female will abort the fetus (45000 is value of mass used to indicate abortion)
    fetus_calf[k*T+t+1]=0;
    }else{
    if(fetus_calf[k*T+t]!=0){                         //if the foetus is still alive
     int xi=(x[k*T+t]-(x_abo-int_x))/int_x;
     float xl=xs_p[xi];  //find closest discretised value of blubber
     st[k*T+t]=resDp[xi*locs+xs_p.size()*locs*i[k*T+t]+xs_p.size()*(locs)*3*t+l[k*T+t]];
     if((st[k*T+t]>3)&&(l[k*T+t]!=1)){                      //if some of the strategies have equivalent fitness (i.e. strategy>3):
      if(st[k*T+t]==4){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(1,3,1),1,TRUE); st[k*T+t]=sts[0];}      //pick one of the three strategies randomly
      if(st[k*T+t]==5){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(1,2,1),1,TRUE); st[k*T+t]=sts[0]; }     //pick one of the two strategies randomly
      if(st[k*T+t]==6){Rcpp::NumericVector ists(2); ists[0]=1; ists[1]=3; Rcpp::NumericVector sts=RcppArmadillo::sample(ists,1,TRUE); st[k*T+t]=sts[0];}     //pick one of the two strategies randomly
      if(st[k*T+t]==7){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(2,3,1),1,TRUE); st[k*T+t]=sts[0];}      //pick one of the two strategies randomly
      }
     if((st[k*T+t]>4)&&(l[k*T+t]==1)){                      //if some of the strategies have equivalent fitness (i.e. strategy>3):
      if(st[k*T+t]==5){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(1,4,1),1,TRUE); st[k*T+t]=sts[0];}   //pick one of the four strategies randomly
      if(st[k*T+t]==6){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(1,3,1),1,TRUE); st[k*T+t]=sts[0];}   //pick one of the three strategies randomly
      if(st[k*T+t]==7){Rcpp::NumericVector ists(3); ists[0]=1; ists[1]=2; ists[2]=4; Rcpp::NumericVector sts=RcppArmadillo::sample(ists,1,TRUE); st[k*T+t]=sts[0];}   //pick one of the three strategies randomly
      if(st[k*T+t]==8){Rcpp::NumericVector ists(3); ists[0]=1; ists[1]=3; ists[2]=4; Rcpp::NumericVector sts=RcppArmadillo::sample(ists,1,TRUE); st[k*T+t]=sts[0];}   //pick one of the three strategies randomly
      if(st[k*T+t]==9){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(2,4,1),1,TRUE); st[k*T+t]=sts[0];}     //pick one of the three strategies randomly
      if(st[k*T+t]==10){Rcpp::NumericVector ists(2); ists[0]=1; ists[1]=3; Rcpp::NumericVector sts=RcppArmadillo::sample(ists,1,TRUE); st[k*T+t]=sts[0];}    //pick one of the two strategies randomly
      if(st[k*T+t]==11){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(1,2,1),1,TRUE); st[k*T+t]=sts[0];}    //pick one of the two strategies randomly
      if(st[k*T+t]==12){Rcpp::NumericVector ists(2); ists[0]=1; ists[1]=4; Rcpp::NumericVector sts=RcppArmadillo::sample(ists,1,TRUE); st[k*T+t]=sts[0];}    //pick one of the two strategies randomly
      if(st[k*T+t]==13){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(2,3,1),1,TRUE); st[k*T+t]=sts[0];}    //pick one of the two strategies randomly
      if(st[k*T+t]==14){Rcpp::NumericVector ists(2); ists[0]=2; ists[1]=4; Rcpp::NumericVector sts=RcppArmadillo::sample(ists,1,TRUE); st[k*T+t]=sts[0];}    //pick one of the two strategies randomly
      if(st[k*T+t]==15){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(3,4,1),1,TRUE); st[k*T+t]=sts[0];}    //pick one of the two strategies randomly
      }

     Rcpp::NumericVector prd(1);prd[0]=dist_locs[l[k*T+t]];
     Rcpp::NumericVector dis=vecrbinom(1,prd);
     if((st[k*T+t]==1) && (dis[0]==1)){  //if optimal behaviour is to stay but location is disturbed that day
      if(l[k*T+t]!=1){
       disturbed[k*T+t]=1;               //they are disturbed on that day
       Rcpp::NumericVector prr(1);prr[0]=responsive[k];
       Rcpp::NumericVector res=vecrbinom(1,prr);
       if(res[0]==1){
        resp[k*T+t]=resDp2[xi*locs+xs_p.size()*locs*i[k*T+t]+xs_p.size()*(locs)*3*t+l[k*T+t]];   //if disturbed from location 3 onwards, they can respond by picking second best strategy
        if(resp[k*T+t]==1){               //if second best choice is again to stay (because of random pick above), pick another strategy randomly
         Rcpp::NumericVector rss=RcppArmadillo::sample(sequence(2,3,1),1,TRUE);
         resp[k*T+t]=rss[0];
         }
        }else{
        resp[k*T+t]=1;     //response depends on responsiveness (if responsive, she leaves)
        }
       st[k*T+t]=resp[k*T+t];                                //their behaviour is adjusted in response to disturbance
       }
      if(l[k*T+t]==1){
       disturbed[k*T+t]=1;                                   //they are disturbed on that day
       Rcpp::NumericVector prr(1);prr[0]=responsive[k];
       Rcpp::NumericVector res=vecrbinom(1,prr);
       if(res[0]==1){
        resp[k*T+t]=resDp2[xi*locs+xs_p.size()*locs*i[k*T+t]+xs_p.size()*(locs)*3*t+l[k*T+t]];    //if disturbed from loc 3 onwards, they can respond by picking second best strategy
        if(resp[k*T+t]==1){                         //if second best choice is again to stay (because of random pick above), pick another strategy randomly
         Rcpp::NumericVector rss=RcppArmadillo::sample(sequence(2,4,1),1,TRUE);
         resp[k*T+t]=rss[0];
         }
        }else{
        resp[k*T+t]=1;     //response depends on responsiveness (if responsive, she leaves)
        }
       st[k*T+t]=resp[k*T+t];                                 //their behaviour is adjusted in response to disturbance
       }
      }

     if((l[k*T+t]>=(locs-1)) && (st[k*T+t]==2)){      //locs-1 because C++
      st[k*T+t]=1;                                    //if in the last possible location, she will not move
      }
     if(st[k*T+t]==1){                                //if she decides to stay and feed/rest
      consec[k]=consec[k]+1;                          //increase number of consecutive days in that patch by 1
      l[k*T+t+1]=l[k*T+t];                            //location doesnt change
      if (consec[k]<=degrade){                        //if she has been in that patch for less than 3 weeks
       i[k*T+t+1]=i[k*T+t];                           //patch type doesnt change
       }else{
       consec[k]=0;                                   //reset counter for consecutive days in patch (since patch changes)
       i[k*T+t+1]=2;                                  //and patch type gets degraded (type 3, 2 in C++)
       }

      Rcpp::NumericVector yju(30);
      Rcpp::NumericVector pju(30);
      for (int yjj=0; yjj<30; ++yjj){
       yju[yjj]=yj[i[k*T+t]*30 + 3*30*l[k*T+t] + 3*30*(locs+1)*t + yjj];
       pju[yjj]=pj[i[k*T+t]*30 + 3*30*l[k*T+t] + 3*30*(locs+1)*t + yjj];
       }
      float prey=yju[oneMultinom(pju)];         //pick a random amount of food found on patch
      float for_hoursu;
      for_hoursu=for_hours[l[k*T+t]*(T+1)+t];
      Rcpp::NumericVector preys(2);
      preys[0]=prey;
      preys[1]=prey;
      Rcpp::NumericVector cfs=cf2(x[k*T+t]+Wc[t+timeS], lean, raamr, preys, w, beta, psi, tc, l[k*T+t], t, for_hoursu, disturbed[k*T+t], severity);
      Rcpp::NumericVector crs=cr2(x[k*T+t]+Wc[t+timeS], lean, preys, w, beta, psi, tc, l[k*T+t], t, for_hoursu, disturbed[k*T+t], severity, scenarion[38]);
      Rcpp::NumericVector foods=food2(preys, w, beta, psi, tc, l[k*T+t], t, for_hoursu, rho, A, disturbed[k*T+t], severity);
      x[k*T+t+1]=  x[k*T+t]+(- cfs[0] - crs[0] + foods[0] - ag(t+timeS,Wc_d,cost_kg,propg,Chg_tot))/u;                    //update blubber accordingly
      if(x[k*T+t+1]>C) x[k*T+t+1]=C;                   //if blubber greater than max, set to max (C)
      if(x[k*T+t+1]>=x_abo){
       fetus_calf[k*T+t+1]=1;
       }
       else
       {
       fetus_calf[k*T+t+1]=0;
       }//if blubber falls below x_abo, fetus is aborted
      }
     if(st[k*T+t]==2){                                //if female decides to move on
      consec[k]=0;                                    //reset counter for consecutive days
      x[k*T+t+1]=x[k*T+t] + (- cot(x[k*T+t]+Wc[t+timeS],lean, scenarion[39])*d*(x[k*T+t]+lean+Wc[t+timeS])- ag(t+timeS,Wc_d,cost_kg,propg,Chg_tot))/u;
      l[k*T+t+1]=l[k*T+t]+1;                          //location increases by 1
      Rcpp::NumericVector lambda_f(3);
      for (int la=0; la<3; ++la){
       lambda_f[la]=lambda[l[k*T+t+1]*3 + (locs+1)*3*(t+1) + la]; //lambda depends on location and time
       }
      i[k*T+t+1]=oneMultinom(lambda_f);                           //pick a patch type reached in the new location
      if(x[k*T+t+1]>=x_abo){
       fetus_calf[k*T+t+1]=1;
       }
       else
       {
       fetus_calf[k*T+t+1]=0;
       }//if blubber falls below x_abo, fetus is aborted
      }
     if((st[k*T+t]==3)||((l[k*T+t]==1)&&(st[k*T+t]==4))){         //if female decides to move back (or within in case of breeding ground)
      consec[k]=0;                                                //reset counter for consecutive days
      x[k*T+t+1]=x[k*T+t] + (- cot(x[k*T+t]+Wc[t+timeS],lean, scenarion[39])*d*(x[k*T+t]+lean+Wc[t+timeS])- ag(t+timeS,Wc_d,cost_kg,propg,Chg_tot))/u;
      if((l[k*T+t]==0)||((l[k*T+t]==1)&&(st[k*T+t]==4))){         //if in breeding ground
       l[k*T+t+1]=l[k*T+t];                                       //she moves within
       }else{
       l[k*T+t+1]=l[k*T+t]-1;                                     //otherwise location decreases by 1
       }
      Rcpp::NumericVector lambda_f(3);
      for (int la=0; la<3; ++la){
       lambda_f[la]=lambda[l[k*T+t+1]*3 + (locs+1)*3*(t+1) + la]; //lambda depends on location and time
       }
      i[k*T+t+1]=oneMultinom(lambda_f);
      if(x[k*T+t+1]>=x_abo){
       fetus_calf[k*T+t+1]=1;
       }
       else
       {
       fetus_calf[k*T+t+1]=0;
       }//if blubber falls below x_abo, fetus is aborted
      }
     }else{
     fetus_calf[k*T+t+1]=0;  //if the foetus is dead, it stays dead
     x[k*T+t+1]=45000;       //to indicate the foetus has been aborted
     }
    }
   }

   //LINKING STEP: t=243 (day before birth of the calf), 242 in c++
   if(t==242){
    if(x[k*T+t]<x_abo){    //if her mass is below the threshold for abortion or if she is not in the breeding ground at the time of birth, the fetus dies
     x[k*T+t+1]=45000;
     fetus_calf[k*T+t+1]=0;
     }else{
     if(fetus_calf[k*T+t]!=0){
      st[k*T+t]=0;
      i[k*T+t+1]=i[k*T+t];
      l[k*T+t+1]=l[k*T+t];
      float prey=0;       //female doesnt feed on day she gives birth
      float for_hoursu;
      for_hoursu=for_hours[l[k*T+t]*(T+1)+t];
      Rcpp::NumericVector preys(2);
      preys[0]=prey;
      preys[1]=prey;
      Rcpp::NumericVector crs=cr2(x[k*T+t]+Wc[t+timeS], lean, preys, w, beta, psi, tc, l[k*T+t], t, for_hoursu, disturbed[k*T+t], severity, scenarion[38]);
      x[k*T+t+1]=x[k*T+t]+(-crs[0] - ag(t+timeS,Wc_d,cost_kg,propg,Chg_tot))/u;
      if(x[k*T+t+1]>=x_abo){
       fetus_calf[k*T+t+1]=1;
       }
       else
       {
       fetus_calf[k*T+t+1]=0;
       }//if blubber falls below x_abo, fetus is aborted
      if((l[k*T+t]!=0)&&(l[k*T+t]!=1)){
       x[k*T+t+1]=45000;
       fetus_calf[k*T+t+1]=0;
       }
      }else{
      fetus_calf[k*T+t+1]=0;
      x[k*T+t+1]=45000;
     }
    }
   }

   //lactation
   if(t>242){   //242 in c++
    if(x[k*T+t]<=xc){       //if female blubber is equal or below the critical mass for death, the female dies but the calf can survive depending on current blubber %
     x[k*T+t+1]=xc;
     if(std::isnan(fetus_calf[k*T+(t+1)])){
      float Erem=Er*(T-t)/u;                                 //blubber required to survive and grow up to 7 months (without milk from mother), i.e. for T-t days remaining, until the moment when calf can autonomously feed
      float bm=leand[t-t_il]*m[k*T+t]/(1-m[k*T+t]);          //current blubber mass, derived from % of blubber and lean mass in previous time step
      float fms=(bm-Erem)/(leand[leand.size()-1]+(bm-Erem)); //final % blubber mass, given current mass and mass required to survive and grow up to 7 months
      if(fms<0) fms=0;
      for (int tt=(t+1); tt<T; ++tt) {fetus_calf[k*T+tt]=0; m[k*T+tt]=fms;}
      }
     }else{
     if(fetus_calf[k*T+t]!=0){
      int xi=(x[k*T+t]-xc)/int_x;
      float xl=xs_l[xi];  //find closest discretised value of blubber
      if(xl==xc){
       xl=xc+int_x;       //when above the critical mass, the female doesnt die - use the following mass bin
       xi=xi+1;
       }
      int mi=(m[k*T+t]-mc)/int_m;
      float ml=ms[mi];
      if(ml==mc){
       ml=mc+int_m;       //when above the critical % mass, the calf doesnt die - use the following mass bin
       mi=mi+1;
       }
      st[k*T+t]=resDl[xi*ms.size()+xs_l.size()*ms.size()*l[k*T+t]+xs_l.size()*ms.size()*(locs)*i[k*T+t]+xs_l.size()*ms.size()*(locs)*3*(t-t_il)+mi];

      if((st[k*T+t]>3)&&(l[k*T+t]!=1)){                      //if some of the strategies have equivalent fitness (i.e. strategy>3):
       if(st[k*T+t]==4){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(1,3,1),1,TRUE); st[k*T+t]=sts[0];}      //pick one of the three strategies randomly
       if(st[k*T+t]==5){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(1,2,1),1,TRUE); st[k*T+t]=sts[0]; }     //pick one of the two strategies randomly
       if(st[k*T+t]==6){Rcpp::NumericVector ists(2); ists[0]=1; ists[1]=3; Rcpp::NumericVector sts=RcppArmadillo::sample(ists,1,TRUE); st[k*T+t]=sts[0];}     //pick one of the two strategies randomly
       if(st[k*T+t]==7){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(2,3,1),1,TRUE); st[k*T+t]=sts[0];}      //pick one of the two strategies randomly
       }
      if((st[k*T+t]>4)&&(l[k*T+t]==1)){                      //if some of the strategies have equivalent fitness (i.e. strategy>3):
       if(st[k*T+t]==5){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(1,4,1),1,TRUE); st[k*T+t]=sts[0];}   //pick one of the four strategies randomly
       if(st[k*T+t]==6){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(1,3,1),1,TRUE); st[k*T+t]=sts[0];}   //pick one of the three strategies randomly
       if(st[k*T+t]==7){Rcpp::NumericVector ists(3); ists[0]=1; ists[1]=2; ists[2]=4; Rcpp::NumericVector sts=RcppArmadillo::sample(ists,1,TRUE); st[k*T+t]=sts[0];}   //pick one of the three strategies randomly
       if(st[k*T+t]==8){Rcpp::NumericVector ists(3); ists[0]=1; ists[1]=3; ists[2]=4; Rcpp::NumericVector sts=RcppArmadillo::sample(ists,1,TRUE); st[k*T+t]=sts[0];}   //pick one of the three strategies randomly
       if(st[k*T+t]==9){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(2,4,1),1,TRUE); st[k*T+t]=sts[0];}   //pick one of the three strategies randomly
       if(st[k*T+t]==10){Rcpp::NumericVector ists(2); ists[0]=1; ists[1]=3; Rcpp::NumericVector sts=RcppArmadillo::sample(ists,1,TRUE); st[k*T+t]=sts[0];}    //pick one of the two strategies randomly
       if(st[k*T+t]==11){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(1,2,1),1,TRUE); st[k*T+t]=sts[0];}  //pick one of the two strategies randomly
       if(st[k*T+t]==12){Rcpp::NumericVector ists(2); ists[0]=1; ists[1]=4; Rcpp::NumericVector sts=RcppArmadillo::sample(ists,1,TRUE); st[k*T+t]=sts[0];}    //pick one of the two strategies randomly
       if(st[k*T+t]==13){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(2,3,1),1,TRUE); st[k*T+t]=sts[0];}  //pick one of the two strategies randomly
       if(st[k*T+t]==14){Rcpp::NumericVector ists(2); ists[0]=2; ists[1]=4; Rcpp::NumericVector sts=RcppArmadillo::sample(ists,1,TRUE); st[k*T+t]=sts[0];}    //pick one of the two strategies randomly
       if(st[k*T+t]==15){Rcpp::NumericVector sts=RcppArmadillo::sample(sequence(3,4,1),1,TRUE); st[k*T+t]=sts[0];}  //pick one of the two strategies randomly
       }

     Rcpp::NumericVector prd(1);prd[0]=dist_locs[l[k*T+t]];
     Rcpp::NumericVector dis=vecrbinom(1,prd);
     if((st[k*T+t]==1) && (dis[0]==1)){  //if optimal behaviour is to stay but location is disturbed that day
      if(l[k*T+t]>1){
       disturbed[k*T+t]=1;                                  //they are disturbed on that day
       Rcpp::NumericVector prr(1);prr[0]=responsive[k];
       Rcpp::NumericVector res=vecrbinom(1,prr);
       if(res[0]==1){
        resp[k*T+t]=resDl2[xi*ms.size()+xs_l.size()*ms.size()*l[k*T+t]+xs_l.size()*ms.size()*(locs)*i[k*T+t]+xs_l.size()*ms.size()*(locs)*3*(t-t_il)+mi];   //if disturbed from loc 3 onwards, they can respond by picking second best strategy
        if(resp[k*T+t]==1){                         //if second best choice is again to stay (because of random pick above), pick another strategy randomly
         Rcpp::NumericVector rss=RcppArmadillo::sample(sequence(2,3,1),1,TRUE);
         resp[k*T+t]=rss[0];
         }
        }else{
        resp[k*T+t]=1;     //response depends on responsiveness (if responsive, she leaves)
        }
       st[k*T+t]=resp[k*T+t];                                //their behaviour is adjusted in response to disturbance
       }
      if(l[k*T+t]==1){
       disturbed[k*T+t]=1;                                   //they are disturbed on that day
       Rcpp::NumericVector prr(1);prr[0]=responsive[k];
       Rcpp::NumericVector res=vecrbinom(1,prr);
       if(res[0]==1){
        resp[k*T+t]=resDl2[xi*ms.size()+xs_l.size()*ms.size()*l[k*T+t]+xs_l.size()*ms.size()*(locs)*i[k*T+t]+xs_l.size()*ms.size()*(locs)*3*(t-t_il)+mi];  //if disturbed from loc 3 onwards, they can respond by picking second best strategy
        if(resp[k*T+t]==1){                         //if second best choice is again to stay (because of random pick above), pick another strategy randomly
         Rcpp::NumericVector rss=RcppArmadillo::sample(sequence(2,4,1),1,TRUE);
         resp[k*T+t]=rss[0];
         }
        }else{
        resp[k*T+t]=1;     //response depends on responsiveness (if responsive, she leaves)
        }
       if((t<daysize) && (resp[k*T+t]==2)){
        Rcpp::NumericVector rss=RcppArmadillo::sample(sequence(3,4,1),1,TRUE);
        resp[k*T+t]=rss[0];                        //if second best strategy is move forward but they are constrained to do so on that day, pick one of the other two at random
        }                                          //before the calf can travel, she will only move within breeding ground when responding to disturbance
       st[k*T+t]=resp[k*T+t];                      //their behaviour is adjusted in response to disturbance
       }
      if(l[k*T+t]==0){
       disturbed[k*T+t]=1;                         //they are disturbed on that day
       Rcpp::NumericVector prr(1);prr[0]=responsive[k];
       Rcpp::NumericVector res=vecrbinom(1,prr);
       if(res[0]==1){
        resp[k*T+t]=resDl2[xi*ms.size()+xs_l.size()*ms.size()*l[k*T+t]+xs_l.size()*ms.size()*(locs)*i[k*T+t]+xs_l.size()*ms.size()*(locs)*3*(t-t_il)+mi];   //if disturbed from loc 3 onwards, they can respond by picking second best strategy
        if(resp[k*T+t]==1){                         //if second best choice is again to stay (because of random pick above), pick another strategy randomly
         Rcpp::NumericVector rss=RcppArmadillo::sample(sequence(2,3,1),1,TRUE);
         resp[k*T+t]=rss[0];
         }
        }else{
        resp[k*T+t]=1;     //response depends on responsiveness (if responsive, she leaves)
        }
       //resp[k*T+t]<-ifelse(rbinom(1,1,responsive[k]*condeff(x[k*T+t])*qualeff(mean(yj[i[k*T+t],,l[k*T+t]*T+t])))==1,sample(c(2,3),1),1)//make response depend not only on responsiveness but also condition and/or quality
       if((t<daysize) && (resp[k*T+t]==2)){
        resp[k*T+t]=3;             //if second best strategy is to move forward but they are constrained on that day, they move within
        }                          //before the calf can travel, she will only move within breeding ground when responding to disturbance
       st[k*T+t]=resp[k*T+t];      //behaviour is adjusted in response to disturbance
       }
      }

      if((l[k*T+t]>=(locs-1)) && (st[k*T+t]==2)){
       st[k*T+t]=1;
       }
      if(st[k*T+t]==1){
       consec[k]=consec[k]+1;                        //increase number of consecutive days in that patch by 1
       if (consec[k]<= degrade){                     //if she has been in that patch for less than 3 weeks
        i[k*T+t+1]=i[k*T+t];                         //patch type doesnt change
        }else{
        consec[k]=0;                                 //reset counter for consecutive days in patch (since patch changes)
        i[k*T+t+1]=2;                                //and patch type gets degraded (2 in c++)
        }
       l[k*T+t+1]=l[k*T+t];
       Rcpp::NumericVector yju(30);
       Rcpp::NumericVector pju(30);
       for (int yjj=0; yjj<30; ++yjj){
        yju[yjj]=yj[i[k*T+t]*30 + 3*30*l[k*T+t] + 3*30*(locs+1)*t + yjj];
        pju[yjj]=pj[i[k*T+t]*30 + 3*30*l[k*T+t] + 3*30*(locs+1)*t + yjj];
        }
       float prey=yju[oneMultinom(pju)];         //pick a random amount of food found on patch
       float for_hoursu;
       for_hoursu=for_hours[l[k*T+t]*(T+1)+t];
       Rcpp::NumericVector preys(2);
       preys[0]=prey;
       preys[1]=prey;
       Rcpp::NumericVector cfs=cf2(x[k*T+t], lean, raamr, preys, w, beta, psi, tc, l[k*T+t], t, for_hoursu, disturbed[k*T+t], severity);
       Rcpp::NumericVector crs=cr2(x[k*T+t], lean, preys, w, beta, psi, tc, l[k*T+t], t, for_hoursu, disturbed[k*T+t], severity, scenarion[38]);
       Rcpp::NumericVector foods=food2(preys, w, beta, psi, tc, l[k*T+t], t, for_hoursu, rho, A, disturbed[k*T+t], severity);
       float test1=x[k*T+t]+(- cfs[0] - crs[0] + foods[0])/u - x_lac;                    //update blubber accordingly
       float test2=(x[k*T+t]-x_lac)*u - cfs[0] - crs[0]+ foods[0] - Mmax*me/E;           //update blubber accordingly
       if(test1<=0){
        x[k*T+t+1]= x[k*T+t]+(- cfs[0] - crs[0] + foods[0])/u;                           //update blubber accordingly
        float milk = 0;
        m[k*T+t+1]= blub_perc(milk,t,m[k*T+t],me,Er,u,t_il,leand);
        if(m[k*T+t+1]>mC){
         x[k*T+t+1]=x[k*T+t+1]+extra_m(m[k*T+t+1],t,t_il,mC,E,leand);
         m[k*T+t+1]=mC;
         }
        }
       if((test1>0) && (test2<0)){
        x[k*T+t+1] = x_lac;
        float milk = ((x[k*T+t]-x_lac)*u - cfs[0] - crs[0] + foods[0])*E/me;
        m[k*T+t+1] = blub_perc(milk,t,m[k*T+t],me,Er,u,t_il,leand);
        if(m[k*T+t+1]>mC){
         x[k*T+t+1]=x[k*T+t+1]+extra_m(m[k*T+t+1],t,t_il,mC,E,leand);
         m[k*T+t+1]=mC;
         }
        }
       if((test1>0) && (test2>=0)){
        x[k*T+t+1] = x[k*T+t]+ (- cfs[0] - crs[0] + foods[0] - Mmax*me/E)/u;
        float milk = Mmax;
        m[k*T+t+1] = blub_perc(milk,t,m[k*T+t],me,Er,u,t_il,leand);
        if(m[k*T+t+1]>mC){
         x[k*T+t+1]=x[k*T+t+1]+extra_m(m[k*T+t+1],t,t_il,mC,E,leand);
         m[k*T+t+1]=mC;
         }
        }
       if(x[k*T+t+1]>C){x[k*T+t+1]=C;}
       if(m[k*T+t+1]<=mc){
        for (int tt=(t+1); tt<T; ++tt) fetus_calf[k*T+tt]=0;
        x[k*T+t+1]=45000;
        }
        else
        {
        fetus_calf[k*T+t+1]=1;
        }             //if the % calf blubber mass falls below the threshold for survival, the calf dies
       }
      if(st[k*T+t]==2){
       consec[k]=0;                                  //reset counter for consecutive days
       l[k*T+t+1]=l[k*T+t]+1;
       float test1 = x[k*T+t] - cot(x[k*T+t],lean, scenarion[39])*d*(x[k*T+t]+lean)/u - x_lac;
       float test2 =(x[k*T+t]-x_lac)*u - cot(x[k*T+t],lean, scenarion[39])*d*(x[k*T+t]+lean) - Mmax*me/E;
       if(test1<=0){
        x[k*T+t+1] = x[k*T+t] - cot(x[k*T+t],lean, scenarion[39])*d*(x[k*T+t]+lean)/u;
        float milk = 0;
        m[k*T+t+1] = blub_perc(milk,t,m[k*T+t],me,Er,u,t_il,leand);
        if(m[k*T+t+1]>mC){
         x[k*T+t+1]=x[k*T+t+1]+extra_m(m[k*T+t+1],t,t_il,mC,E,leand);
         m[k*T+t+1]=mC;
         }
        }
       if((test1>0) && (test2<0)){
        x[k*T+t+1] = x_lac;
        float milk = ((x[k*T+t]-x_lac)*u - cot(x[k*T+t],lean, scenarion[39])*d*(x[k*T+t]+lean))*E/me;
        m[k*T+t+1] = blub_perc(milk,t,m[k*T+t],me,Er,u,t_il,leand);
        if(m[k*T+t+1]>mC){
         x[k*T+t+1]=x[k*T+t+1]+extra_m(m[k*T+t+1],t,t_il,mC,E,leand);
         m[k*T+t+1]=mC;
         }
        }
       if((test1>0) && (test2>=0)){
        x[k*T+t+1] = x[k*T+t]+ (-cot(x[k*T+t],lean, scenarion[39])*d*(x[k*T+t]+lean) - Mmax*me/E)/u;
        float milk = Mmax;
        m[k*T+t+1] = blub_perc(milk,t,m[k*T+t],me,Er,u,t_il,leand);
        if(m[k*T+t+1]>mC){
         x[k*T+t+1]=x[k*T+t+1]+extra_m(m[k*T+t+1],t,t_il,mC,E,leand);
         m[k*T+t+1]=mC;
         }
        }
       Rcpp::NumericVector lambda_f(3);
       for (int la=0; la<3; ++la){
        lambda_f[la]=lambda[l[k*T+t+1]*3 + (locs+1)*3*(t+1) + la];                 //lambda depends on location and time
        }
       i[k*T+t+1]=oneMultinom(lambda_f);
       if(m[k*T+t+1]<=mc){
        for (int tt=(t+1); tt<T; ++tt) fetus_calf[k*T+tt]=0;
        x[k*T+t+1]=45000;
        }
        else
        {
        fetus_calf[k*T+t+1]=1;
        }             //if the % calf blubber mass falls below the threshold for survival, the calf dies
       }
      if((st[k*T+t]==3)||((l[k*T+t]==1)&&(st[k*T+t]==4))){
       consec[k]=0;                                      //reset counter for consecutive days
       if((l[k*T+t]==0)||((l[k*T+t]==1)&&(st[k*T+t]==4))){                         //if in breeding ground
        l[k*T+t+1]=l[k*T+t];                             //she stays within
        }else{
        l[k*T+t+1]=l[k*T+t]-1;                           //otherwise location decreases by 1
        }
       float test1 = x[k*T+t] - cot(x[k*T+t],lean, scenarion[39])*d*(x[k*T+t]+lean)/u - x_lac;
       float test2 =(x[k*T+t]-x_lac)*u - cot(x[k*T+t],lean, scenarion[39])*d*(x[k*T+t]+lean) - Mmax*me/E;
       if(test1<=0){
        x[k*T+t+1] = x[k*T+t] - cot(x[k*T+t],lean, scenarion[39])*d*(x[k*T+t]+lean)/u;
        float milk = 0;
        m[k*T+t+1] = blub_perc(milk,t,m[k*T+t],me,Er,u,t_il,leand);
        if(m[k*T+t+1]>mC){
         x[k*T+t+1]=x[k*T+t+1]+extra_m(m[k*T+t+1],t,t_il,mC,E,leand);
         m[k*T+t+1]=mC;
         }
        }
       if((test1>0) && (test2<0)){
        x[k*T+t+1] = x_lac;
        float milk = ((x[k*T+t]-x_lac)*u - cot(x[k*T+t],lean, scenarion[39])*d*(x[k*T+t]+lean))*E/me;
        m[k*T+t+1] = blub_perc(milk,t,m[k*T+t],me,Er,u,t_il,leand);
        if(m[k*T+t+1]>mC){
         x[k*T+t+1]=x[k*T+t+1]+extra_m(m[k*T+t+1],t,t_il,mC,E,leand);
         m[k*T+t+1]=mC;
         }
        }
       if((test1>0) && test2>=0){
        x[k*T+t+1] = x[k*T+t]+ (-cot(x[k*T+t],lean, scenarion[39])*d*(x[k*T+t]+lean) - Mmax*me/E)/u;
        float milk = Mmax;
        m[k*T+t+1] = blub_perc(milk,t,m[k*T+t],me,Er,u,t_il,leand);
        if(m[k*T+t+1]>mC){
         x[k*T+t+1]=x[k*T+t+1]+extra_m(m[k*T+t+1],t,t_il,mC,E,leand);
         m[k*T+t+1]=mC;
         }
        }
       Rcpp::NumericVector lambda_f(3);
       for (int la=0; la<3; ++la){
        lambda_f[la]=lambda[l[k*T+t+1]*3 + (locs+1)*3*(t+1) + la];              //lambda depends on location and time
        }
       i[k*T+t+1]=oneMultinom(lambda_f);
       if(m[k*T+t+1]<=mc){
        for (int tt=(t+1); tt<T; ++tt) fetus_calf[k*T+tt]=0;
        x[k*T+t+1]=45000;
        }
        else
        {
        fetus_calf[k*T+t+1]=1;
        }             //if the % calf blubber mass falls below the threshold for survival, the calf dies
       }
      }else{
      fetus_calf[k*T+t+1]=0;
      if(x[k*T+t]!=45000){
      x[k*T+t+1]=xc;
      }else{
      x[k*T+t+1]=45000;
      }
      }
     }
    }

 } //end of temporal loop
} //end of individual loop


 Rcpp::List ret;
 ret["x"] = x;
 ret["m"] = m;
 ret["l"] = l;
 ret["i"] = i;
 ret["fetus_calf"] = fetus_calf;
 ret["st"] = st;
 ret["disturbed"] = disturbed;
 ret["resp"] = resp;
 return ret;


}
')


##################################################################################Sensitivity scenarios####################################################################################################################################################################################################################################################################################################################
#generate scenarios
npar<-41                               #number of parameters
scenarios<-array(1,dim=c(npar*6,npar)) #array for combinations of parameter values: each line is one combination, where each column contains the multiplicative factor for each parameter (all 1 except 1 per combination)
int<-seq(1,(npar+1)*6,by=6)
for (i in 1:npar){
 scenarios[int[i]:(int[i+1]-1),i]<-c(0.6,0.8,0.9,1.1,1.2,1.4)
 }

#change/remove unrealistic scenarios
scenarios[which(scenarios[,2]==1.4),2]<-1.3   #since 0.73*1.4 is greater than 100%
scenarios[which(scenarios[,20]==1.4),20]<-1.3 #since 0.73*1.4 is greater than 100%
scenarios[which(scenarios[,17]==0.6),17]<-1   #since 0.6*285 is before the birth (0.8 also is, but keep that as minimum, i.e. calf can leave straight away)
scenarios[which(scenarios[,18]==1.1),18]<-1   #since 0.94*1.1 is greater than 100% (and lean mass at birth can be max 0.94, otherwise not enough blubber to survive)
scenarios[which(scenarios[,18]==1.2),18]<-1   #since 0.94*1.2 is greater than 100%
scenarios[which(scenarios[,18]==1.4),18]<-1   #since 0.94*1.2 is greater than 100%

sens_res<-array(NA,dim=c(dim(scenarios)[1],30))

#######################################################run code###########################################################################################################################################################################################################################################################################################################################################################
for (sce in 1:dim(scenarios)[1]){

 scenarion<-scenarios[sce,]  #be careful because when this is fed to a C++ function: array indices should be -1 (but not in R)

 #####General parameters#####
 T<-454
 locs<-36
 feed_g<-15
 ni<-200    #number of individuals simulated in forward iteration
 nsim<-100  #number of times the forward iteration is repeated

 #####################################SDP runs##############################################

 #####Standard SDP#####
 #external inputs:
 ###Astronomical twilight###
 for_hours<-read.csv("twilight_times_byLoc_forModel.csv",header=T)              #total number of hours available for foraging during a day (not feeding at night)
 for_hours<-rbind(for_hours[1,],for_hours)
 for_hoursn<-t(for_hours)
 for_hoursn2_1<-c(for_hoursn)*scenarion[33] #feed this one to C++
 ###Parameters of the patches###
 #Krill density and seasonal/spatial variations
 mdens<-rgamma(1000000,2.5*scenarion[34],5.5*scenarion[35]) #this is the max density possible (in California Current, in summer, for a swarm of E. pacifica + T. spinifera)
 mdens<-mdens[mdens<quantile(mdens,probs=0.999)]
 lag<-90*scenarion[37]
 gcal_lag<-121+lag-74
 upw<-read.csv("UpwellingIndex1.csv",header=T)
 gcal<-data.frame(julday=0:365,loc=0,pr=0,se=0,pr_scaled=c(upw$pr_scaled[upw$loc==unique(upw$loc)[1]][gcal_lag:366],upw$pr_scaled[upw$loc==unique(upw$loc)[1]][1:(gcal_lag-1)]))
 upw<-rbind(gcal,upw) #add curve for Gulf of California
 upw$pr_scaled[upw$pr_scaled<0]<-0
 for (i in unique(upw$loc)[c(1,6:11)]){
  upw[upw$loc==i,]<-rbind(upw[upw$loc==i,][(366-lag+1):366,],upw[upw$loc==i,][1:(366-lag),])
  }
 coords<-read.csv("locs_100km.csv",header=T)
 coords<-rbind(c(0,0),coords) #add extra line
 yj<-array(0,dim=c(3,30,locs+1,T+1))
 pj<-array(1/30,dim=c(3,30,locs+1,T+1))
 rsize<-0.6*scenarion[36]
 breaks<-cut(mdens*rsize,30)
 yj[1,1:30,1:(feed_g-1),]<-tapply(mdens*rsize,breaks,mean)
 pj[1,1:30,1:(feed_g-1),]<-tapply(mdens*rsize,breaks,length)/length(mdens)
 breaks<-cut(mdens,30)
 yj[2,1:30,feed_g:locs,]<-tapply(mdens,breaks,mean)
 pj[2,1:30,feed_g:locs,]<-tapply(mdens,breaks,length)/length(mdens)
 patch<-1
 for (i in 1:locs){
  if(i>=feed_g){
   patch<-2
   }
  cl<-unique(upw$loc)[which.min(abs(coords[i,2]-unique(upw$loc)))]
  scaling<-upw$pr_scaled[upw$loc==cl][c(91:365,1:180)]
  yj[patch,1:30,i,]<-t(t(yj[patch,1:30,i,])*scaling)
  }
 yjn<-aperm(yj,c(2,1,3,4))
 yjn2_1<-c(yjn) #feed this one to C++
 pjn<-aperm(pj,c(2,1,3,4))
 pjn2_1<-c(pjn) #feed this one to C++
 #Probabilities to encounter patches of different type when travelling in different macroareas (constant, but different patch types)
 lambda<-array(0,dim=c(locs+1,3,T+1))
 lambdatmp1<-0.6*scenarion[38]
 lambdatmp2<-0.7*scenarion[38]
 lambda[1:(feed_g-1),1,]<-lambdatmp1
 lambda[1:(feed_g-1),3,]<-1-lambdatmp1
 lambda[feed_g:locs,2,]<-lambdatmp2
 lambda[feed_g:locs,3,]<-1-lambdatmp2
 lambda[locs+1,3,]<-1   #outside modelled range
 lambdan<-aperm(lambda,c(2,1,3))
 lambdan2_1<-c(lambdan) #feed this one to C++
 runS_list<-SDP(yjn2_1,pjn2_1,for_hoursn2_1,lambdan2_1,scenarion)
 nam<-paste("BW_SensSce_",sce,"_SDP_stdEnv.RData",sep="")
 save(runS_list,file=nam)

 #####SDP for ENSO scenario#####
 #external inputs
 ###Astronomical twilight###
 for_hours<-read.csv("twilight_times_byLoc_forModel.csv",header=T)              #total number of hours available for foraging during a day (not feeding at night)
 for_hours<-rbind(for_hours[1,],for_hours)
 for_hoursn<-t(for_hours)
 for_hoursn2_2<-c(for_hoursn)*scenarion[33] #feed this one to C++
 ###Parameters of the patches###
 #Krill density and seasonal/spatial variations
 mdens<-rgamma(1000000,2.5*scenarion[34],5.5*scenarion[35]) #this is the max density possible (in California Current, in summer, for a swarm of E. pacifica + T. spinifera)
 mdens<-mdens[mdens<quantile(mdens,probs=0.999)]
 lag<-90*scenarion[37]
 gcal_lag<-121+lag-74
 upw<-read.csv("UpwellingIndex1.csv",header=T)
 gcal<-data.frame(julday=0:365,loc=0,pr=0,se=0,pr_scaled=c(upw$pr_scaled[upw$loc==unique(upw$loc)[1]][gcal_lag:366],upw$pr_scaled[upw$loc==unique(upw$loc)[1]][1:(gcal_lag-1)]))
 upw<-rbind(gcal,upw) #add curve for Gulf of California
 upw$pr_scaled[upw$pr_scaled<0]<-0
 for (i in unique(upw$loc)[c(1,6:11)]){
  upw[upw$loc==i,]<-rbind(upw[upw$loc==i,][(366-lag+1):366,],upw[upw$loc==i,][1:(366-lag),])
  }
 coords<-read.csv("locs_100km.csv",header=T)
 coords<-rbind(c(0,0),coords) #add extra line
 yj<-array(0,dim=c(3,30,locs+1,T+1))
 pj<-array(1/30,dim=c(3,30,locs+1,T+1))
 rsize<-0.6*scenarion[36]
 breaks<-cut(mdens*rsize,30)
 yj[1,1:30,1:(feed_g-1),]<-tapply(mdens*rsize,breaks,mean)
 pj[1,1:30,1:(feed_g-1),]<-tapply(mdens*rsize,breaks,length)/length(mdens)
 breaks<-cut(mdens,30)
 yj[2,1:30,feed_g:locs,]<-tapply(mdens,breaks,mean)
 pj[2,1:30,feed_g:locs,]<-tapply(mdens,breaks,length)/length(mdens)
 for (i2 in 1:locs){
  if(i2>=feed_g & i2<=28){scal_Nino<-0.3}else{scal_Nino<-1}  #in southern and central California current, productivity is lower during El Nino conditions (additional scaling - reduction in production by 70% (0.3) according to Chavez et al 2002)
  cl<-unique(upw$loc)[which.min(abs(coords[i2,2]-unique(upw$loc)))]
  scaling<-upw$pr_scaled[upw$loc==cl][c(91:365,1:180)]
  yj[1,1:30,i2,]<-t(t(yj[1,1:30,i2,])*scaling*scal_Nino)
  yj[2,1:30,i2,]<-t(t(yj[2,1:30,i2,])*scaling*scal_Nino)
  }
 yjn<-aperm(yj,c(2,1,3,4))
 yjn2_2<-c(yjn) #feed this one to C++
 pjn<-aperm(pj,c(2,1,3,4))
 pjn2_2<-c(pjn) #feed this one to C++
 #Probabilities to encounter patches of different type when travelling in different macroareas (constant, but different patch types)
 maxNS<-28 #max location at which N. simplex available
 red2<-0.3 #reduction in availability of T.spinifera and E.pacifica in affected range
 lambda<-array(0,dim=c(locs+1,3,T+1))
 lambdatmp1<-0.6*scenarion[38]
 lambdatmp2<-0.7*scenarion[38]
 lambda[1:(feed_g-1),1,]<-lambdatmp1
 lambda[1:(feed_g-1),3,]<-1-lambdatmp1
 lambda[feed_g:maxNS,2,]<-lambdatmp2*red2                  #in region where range of N.simplex shifts
 lambda[feed_g:maxNS,3,]<-1-lambdatmp2                     #in region where range of N.simplex shifts
 lambda[feed_g:maxNS,1,]<-1-(1-lambdatmp2)-lambdatmp2*red2 #in region where range of N.simplex shifts
 lambda[(maxNS+1):locs,2,]<-lambdatmp2
 lambda[(maxNS+1):locs,3,]<-1-lambdatmp2
 lambda[locs+1,3,]<-1 #outside modelled range
 lambdan<-aperm(lambda,c(2,1,3))
 lambdan2_2<-c(lambdan) #feed this one to C++

 runE_list<-SDP(yjn2_2,pjn2_2,for_hoursn2_2,lambdan2_2,scenarion)
 nam<-paste("BW_SensSce_",sce,"_SDP_ENSOEnv.RData",sep="")
 save(runE_list,file=nam)

 #####Environment for 2005-like scenario#####
 ###Parameters of the patches###
 #Krill density and seasonal/spatial variations
 mdens2<-rgamma(1000000,2.5*scenarion[34],5.5*scenarion[35]) #this is the max density possible (in California Current, in summer, for a swarm of E. pacifica + T. spinifera)
 mdens2<-mdens2[mdens2<quantile(mdens2,probs=0.999)]
 lag<-90*scenarion[37]
 gcal_lag<-121+lag-74
 upw<-read.csv("UpwellingIndex1.csv",header=T)
 gcal<-data.frame(julday=0:365,loc=0,pr=0,se=0,pr_scaled=c(upw$pr_scaled[upw$loc==unique(upw$loc)[1]][gcal_lag:366],upw$pr_scaled[upw$loc==unique(upw$loc)[1]][1:(gcal_lag-1)]))
 upw<-rbind(gcal,upw) #add curve for Gulf of California
 upw$pr_scaled[upw$pr_scaled<0]<-0
 for (i2 in unique(upw$loc)[c(1,6:11)]){
  if(i2>=33 & i2<42){nlag<-lag+75}else{nlag<-lag}  #in southern and central California current, productivity delayed by 2-3 months
  upw[upw$loc==i2,]<-rbind(upw[upw$loc==i2,][(366-nlag+1):366,],upw[upw$loc==i2,][1:(366-nlag),])
  }
 coords<-read.csv("locs_100km.csv",header=T)
 coords<-rbind(c(0,0),coords) #add extra line
 yj<-array(0,dim=c(3,30,locs+1,T+1))
 pj<-array(1/30,dim=c(3,30,locs+1,T+1))
 rsize<-0.6*scenarion[36]
 breaks<-cut(mdens2*rsize,30)
 yj[1,1:30,1:(feed_g-1),]<-tapply(mdens2*rsize,breaks,mean)
 pj[1,1:30,1:(feed_g-1),]<-tapply(mdens2*rsize,breaks,length)/length(mdens2)
 breaks<-cut(mdens2,30)
 yj[2,1:30,feed_g:locs,]<-tapply(mdens2,breaks,mean)
 pj[2,1:30,feed_g:locs,]<-tapply(mdens2,breaks,length)/length(mdens2)
 patch<-1
 for (i2 in 1:locs){
  if(i2>=feed_g){
   patch<-2
   }
  cl<-unique(upw$loc)[which.min(abs(coords[i2,2]-unique(upw$loc)))]
  scaling<-upw$pr_scaled[upw$loc==cl][c(91:365,1:180)]
  yj[patch,1:30,i2,]<-t(t(yj[patch,1:30,i2,])*scaling)
  }
 yjn<-aperm(yj,c(2,1,3,4))
 yjn2_3<-c(yjn) #feed this one to C++
 pjn<-aperm(pj,c(2,1,3,4))
 pjn2_3<-c(pjn) #feed this one to C++
 maxR<-28  #region where overall productivity is reduced (i.e. increase in probability of patches without food).
 redP<-0.3 #reduction of overall productivity in affected range (assuming it's the same as in El Nino years, i.e. reduction of 70%)
 lambda<-array(0,dim=c(locs+1,3,T+1))
 lambdatmp1<-0.6*scenarion[38]
 lambdatmp2<-0.7*scenarion[38]
 lambda[1:(feed_g-1),1,]<-lambdatmp1
 lambda[1:(feed_g-1),3,]<-1-lambdatmp1
 lambda[feed_g:locs,2,]<-lambdatmp2
 lambda[feed_g:locs,3,]<-1-lambdatmp2
 lambda[feed_g:maxR,2,]<-lambda[feed_g:maxR,2,]*redP  #in region where overall reduction in productivity
 lambda[feed_g:maxR,3,]<-1-lambda[feed_g:maxR,2,]     #in region where overall reduction in productivity
 lambda[locs+1,3,]<-1 #outside modelled range
 lambdan<-aperm(lambda,c(2,1,3))
 lambdan2_3<-c(lambdan) #feed this one to C++

 #####################################Forward iterations##############################################

 naboB<-c()
 nstaB<-c()
 nmumB<-c()
 naboE1<-c()
 nstaE1<-c()
 nmumE1<-c()
 naboE2<-c()
 nstaE2<-c()
 nmumE2<-c()
 naboD1<-c()
 nstaD1<-c()
 nmumD1<-c()
 naboD2<-c()
 nstaD2<-c()
 nmumD2<-c()
 naboD3<-c()
 nstaD3<-c()
 nmumD3<-c()
 meanRB<-c()
 meanRE1<-c()
 meanRE2<-c()
 meanRD1<-c()
 meanRD2<-c()
 meanRD3<-c()
 CoensBE1<-c()
 CoensBE2<-c()
 CoensBD1<-c()
 CoensBD2<-c()
 CoensBD3<-c()

 for (sim in 1:nsim){
   #Baseline:
   p_dist<-0                                                               #probability of disturbance
   n_dist<-0                                                               #number of locations disturbed
   first<-3                                                                #first location where disturbance can start
   dist_locs<-rep(0,locs)                                                  #create vector of locations to store which ones are disturbed
   dist_locs[sample(first:locs,n_dist)]<-sample(p_dist,n_dist,replace=T)   #set the disturbed locations by sampling n_dist random locations in the feeding ground, and assign one of the possible probabilities
   severity<-1                                                             #severity is reduction in time available to feed
   responsive<-runif(ni,0,0)                                               #draw a level of responsiveness for each female from a beta distribution. 1 they are responsive and flee; 0 they are not responsive and stay
   simB_list<-forwardIT_d_2(ni,resDp=runS_list[[1]], resDl=runS_list[[3]], resDp2=runS_list[[5]], resDl2=runS_list[[6]], yjn2_1, pjn2_1, for_hoursn2_1, lambdan2_1, severity, dist_locs, responsive, scenarion)
   xB<-aperm(array(simB_list[[1]],dim=c(T,ni)),c(2,1))
   mB<-aperm(array(simB_list[[2]],dim=c(T,ni)),c(2,1))
   massB<-mB[,T] #final % blubber mass
   gammaB<-8*scenarion[10]
   m50B<-0.27*scenarion[9]
   pB<-(massB^gammaB)/((massB^gammaB) + (m50B^gammaB))#probability that the kth calf recruits to the population given final % blubber mass
   meanpB<-mean(pB)                                   #mean calf recruitment probability
   varpB<-var(pB)                                     #variances of calf recruitment
   maxsb<-apply(xB[,1:244],1,max)
   naboB<-c(naboB,sum(maxsb==45000))                  #how many individuals go to 45000 before birth (abortions)
   maxsa<-apply(xB[,245:T],1,max)
   nstaB<-c(nstaB,sum(maxsa==45000)-sum(maxsb==45000))#how many individuals go to 45000 after birth (calf starvations)
   mins<-apply(xB,1,min)
   nmumB<-c(nmumB,sum(mins<2500))                     #how many individuals go to xc (mothers' starvations)
   meanRB<-c(meanRB,meanpB)

   #ENSO:
   p_dist<-0                                                               #probability of disturbance
   n_dist<-0                                                               #number of locations disturbed
   first<-3                                                                #first location where disturbance can start
   dist_locs<-rep(0,locs)                                                  #create vector of locations to store which ones are disturbed
   dist_locs[sample(first:locs,n_dist)]<-sample(p_dist,n_dist,replace=T)   #set the disturbed locations by sampling n_dist random locations in the feeding ground, and assign one of the possible probabilities
   severity<-1                                                             #severity is reduction in time available to feed
   responsive<-runif(ni,0,0)                                               #draw a level of responsiveness for each female from a beta distribution. 1 they are responsive and flee; 0 they are not responsive and stay
   simE1_list<-forwardIT_d_2(ni,resDp=runE_list[[1]], resDl=runE_list[[3]], resDp2=runE_list[[5]], resDl2=runE_list[[6]], yjn2_2, pjn2_2, for_hoursn2_2, lambdan2_2, severity, dist_locs, responsive, scenarion)
   xE1<-aperm(array(simE1_list[[1]],dim=c(T,ni)),c(2,1))
   mE1<-aperm(array(simE1_list[[2]],dim=c(T,ni)),c(2,1))
   massE1<-mE1[,T] #final % blubber mass
   gammaE1<-8*scenarion[10]
   m50E1<-0.27*scenarion[9]
   pE1<-(massE1^gammaE1)/((massE1^gammaE1) + (m50E1^gammaE1))#probability that the kth calf recruits to the population given final % blubber mass
   meanpE1<-mean(pE1)                                        #mean calf recruitment probability
   varpE1<-var(pE1)                                          #variances of calf recruitment
   maxsb<-apply(xE1[,1:244],1,max)
   naboE1<-c(naboE1,sum(maxsb==45000))                       #how many individuals go to 45000 before birth (abortions)
   maxsa<-apply(xE1[,245:T],1,max)
   nstaE1<-c(nstaE1,sum(maxsa==45000)-sum(maxsb==45000))     #how many individuals go to 45000 after birth (calf starvations)
   mins<-apply(xE1,1,min)
   nmumE1<-c(nmumE1,sum(mins<2500))                          #how many individuals go to xc (mothers' starvations)
   meanRE1<-c(meanRE1,meanpE1)

   #2005-like:
   p_dist<-0                                                               #probability of disturbance
   n_dist<-0                                                               #number of locations disturbed
   first<-3                                                                #first location where disturbance can start
   dist_locs<-rep(0,locs)                                                  #create vector of locations to store which ones are disturbed
   dist_locs[sample(first:locs,n_dist)]<-sample(p_dist,n_dist,replace=T)   #set the disturbed locations by sampling n_dist random locations in the feeding ground, and assign one of the possible probabilities
   severity<-1                                                             #severity is reduction in time available to feed
   responsive<-runif(ni,0,0)                                               #draw a level of responsiveness for each female from a beta distribution. 1 they are responsive and flee; 0 they are not responsive and stay
   simE2_list<-forwardIT_d_2(ni,resDp=runS_list[[1]], resDl=runS_list[[3]], resDp2=runS_list[[5]], resDl2=runS_list[[6]], yjn2_3, pjn2_3, for_hoursn2_1, lambdan2_3, severity, dist_locs, responsive, scenarion)
   xE2<-aperm(array(simE2_list[[1]],dim=c(T,ni)),c(2,1))
   mE2<-aperm(array(simE2_list[[2]],dim=c(T,ni)),c(2,1))
   massE2<-mE2[,T] #final % blubber mass
   gammaE2<-8*scenarion[10]
   m50E2<-0.27*scenarion[9]
   pE2<-(massE2^gammaE2)/((massE2^gammaE2) + (m50E2^gammaE2))#probability that the kth calf recruits to the population given final % blubber mass
   meanpE2<-mean(pE2)                                        #mean calf recruitment probability
   varpE2<-var(pE2)                                          #variances of calf recruitment
   maxsb<-apply(xE2[,1:244],1,max)
   naboE2<-c(naboE2,sum(maxsb==45000))                       #how many individuals go to 45000 before birth (abortions)
   maxsa<-apply(xE2[,245:T],1,max)
   nstaE2<-c(nstaE2,sum(maxsa==45000)-sum(maxsb==45000))     #how many individuals go to 45000 after birth (calf starvations)
   mins<-apply(xE2,1,min)
   nmumE2<-c(nmumE2,sum(mins<2500))                          #how many individuals go to xc (mothers' starvations)
   meanRE2<-c(meanRE2,meanpE2)

   #Disturbance 1 (localised, intense, no response):
   p_dist<-1                                                               #probability of disturbance
   dist_locs<-rep(0,locs)                                                  #create vector of locations to store which ones are disturbed
   dist_locs[16]<-p_dist
   severity<-0                                                             #severity is reduction in time available to feed
   responsive<-runif(ni,0,0)                                               #draw a level of responsiveness for each female from a beta distribution. 1 they are responsive and flee; 0 they are not responsive and stay
   simD1_list<-forwardIT_d_2(ni,resDp=runS_list[[1]], resDl=runS_list[[3]], resDp2=runS_list[[5]], resDl2=runS_list[[6]], yjn2_1, pjn2_1, for_hoursn2_1, lambdan2_1, severity, dist_locs, responsive, scenarion)
   xD1<-aperm(array(simD1_list[[1]],dim=c(T,ni)),c(2,1))
   mD1<-aperm(array(simD1_list[[2]],dim=c(T,ni)),c(2,1))
   massD1<-mD1[,T] #final % blubber mass
   gammaD1<-8*scenarion[10]
   m50D1<-0.27*scenarion[9]
   pD1<-(massD1^gammaD1)/((massD1^gammaD1) + (m50D1^gammaD1))#probability that the kth calf recruits to the population given final % blubber mass
   meanpD1<-mean(pD1)                                        #mean calf recruitment probability
   varpD1<-var(pD1)                                          #variances of calf recruitment
   maxsb<-apply(xD1[,1:244],1,max)
   naboD1<-c(naboD1,sum(maxsb==45000))                       #how many individuals go to 45000 before birth (abortions)
   maxsa<-apply(xD1[,245:T],1,max)
   nstaD1<-c(nstaD1,sum(maxsa==45000)-sum(maxsb==45000))     #how many individuals go to 45000 after birth (calf starvations)
   mins<-apply(xD1,1,min)
   nmumD1<-c(nmumD1,sum(mins<2500))                          #how many individuals go to xc (mothers' starvations)
   meanRD1<-c(meanRD1,meanpD1)

   #Disturbance 2 (localised, intense, all respond):
   p_dist<-1                                                             #probability of disturbance
   dist_locs<-rep(0,locs)                                                #create vector of locations to store which ones are disturbed
   dist_locs[16]<-p_dist
   severity<-0                                                           #severity is reduction in time available to feed
   responsive<-runif(ni,1,1)                                             #draw a level of responsiveness for each female from a beta distribution. 1 they are responsive and flee; 0 they are not responsive and stay
   simD2_list<-forwardIT_d_2(ni,resDp=runS_list[[1]], resDl=runS_list[[3]], resDp2=runS_list[[5]], resDl2=runS_list[[6]], yjn2_1, pjn2_1, for_hoursn2_1, lambdan2_1, severity, dist_locs, responsive, scenarion)
   xD2<-aperm(array(simD2_list[[1]],dim=c(T,ni)),c(2,1))
   mD2<-aperm(array(simD2_list[[2]],dim=c(T,ni)),c(2,1))
   massD2<-mD2[,T] #final % blubber mass
   gammaD2<-8*scenarion[10]
   m50D2<-0.27*scenarion[9]
   pD2<-(massD2^gammaD2)/((massD2^gammaD2) + (m50D2^gammaD2))#probability that the kth calf recruits to the population given final % blubber mass
   meanpD2<-mean(pD2)                                        #mean calf recruitment probability
   varpD2<-var(pD2)                                          #variances of calf recruitment
   maxsb<-apply(xD2[,1:244],1,max)
   naboD2<-c(naboD2,sum(maxsb==45000))                       #how many individuals go to 45000 before birth (abortions)
   maxsa<-apply(xD2[,245:T],1,max)
   nstaD2<-c(nstaD2,sum(maxsa==45000)-sum(maxsb==45000))     #how many individuals go to 45000 after birth (calf starvations)
   mins<-apply(xD2,1,min)
   nmumD2<-c(nmumD2,sum(mins<2500))                          #how many individuals go to xc (mothers' starvations)
   meanRD2<-c(meanRD2,meanpD2)

   #Disturbance 3 (diffuse, weaker, mean response):
   p_dist<-0.5                                                             #probability of disturbance
   n_dist<-17                                                              #number of locations disturbed
   first<-3                                                                #first location where disturbance can start
   dist_locs<-rep(0,locs)                                                  #create vector of locations to store which ones are disturbed
   dist_locs[sample(first:locs,n_dist)]<-sample(p_dist,n_dist,replace=T)   #set the disturbed locations by sampling n_dist random locations in the feeding ground, and assign one of the possible probabilities
   severity<-0.5                                                           #severity is reduction in time available to feed
   responsive<-rbeta(ni,5,5)                                               #draw a level of responsiveness for each female from a beta distribution. 1 they are responsive and flee; 0 they are not responsive and stay
   simD3_list<-forwardIT_d_2(ni,resDp=runS_list[[1]], resDl=runS_list[[3]], resDp2=runS_list[[5]], resDl2=runS_list[[6]], yjn2_1, pjn2_1, for_hoursn2_1, lambdan2_1, severity, dist_locs, responsive, scenarion)
   xD3<-aperm(array(simD3_list[[1]],dim=c(T,ni)),c(2,1))
   mD3<-aperm(array(simD3_list[[2]],dim=c(T,ni)),c(2,1))
   massD3<-mD3[,T] #final % blubber mass
   gammaD3<-8*scenarion[10]
   m50D3<-0.27*scenarion[9]
   pD3<-(massD3^gammaD3)/((massD3^gammaD3) + (m50D3^gammaD3))#probability that the kth calf recruits to the population given final % blubber mass
   meanpD3<-mean(pD3)                                        #mean calf recruitment probability
   varpD3<-var(pD3)                                          #variances of calf recruitment
   maxsb<-apply(xD3[,1:244],1,max)
   naboD3<-c(naboD3,sum(maxsb==45000))                       #how many individuals go to 45000 before birth (abortions)
   maxsa<-apply(xD3[,245:T],1,max)
   nstaD3<-c(nstaD3,sum(maxsa==45000)-sum(maxsb==45000))     #how many individuals go to 45000 after birth (calf starvations)
   mins<-apply(xD3,1,min)
   nmumD3<-c(nmumD3,sum(mins<2500))                          #how many individuals go to xc (mothers' starvations)
   meanRD3<-c(meanRD3,meanpD3)

   varb<-(varpB+varpE1)/2
   CoensBE1<-c(CoensBE1,(meanpB-meanpE1)/sqrt(varb)) #Cohen's d

   varb<-(varpB+varpE2)/2
   CoensBE2<-c(CoensBE2,(meanpB-meanpE2)/sqrt(varb)) #Cohen's d

   varb<-(varpB+varpD1)/2
   CoensBD1<-c(CoensBD1,(meanpB-meanpD1)/sqrt(varb)) #Cohen's d

   varb<-(varpB+varpD2)/2
   CoensBD2<-c(CoensBD2,(meanpB-meanpD2)/sqrt(varb)) #Cohen's d

   varb<-(varpB+varpD3)/2
   CoensBD3<-c(CoensBD3,(meanpB-meanpD3)/sqrt(varb)) #Cohen's d

   }

  sens_res[sce,]<-c(mean(meanRB),mean(naboB),mean(nstaB),mean(nmumB),
                    mean(meanRE1),mean(naboE1),mean(nstaE1),mean(nmumE1),mean(CoensBE1),
                    mean(meanRE2),mean(naboE2),mean(nstaE2),mean(nmumE2),mean(CoensBE2),
                    mean(meanRD1),mean(naboD1),mean(nstaD1),mean(nmumD1),mean(CoensBD1),
                    mean(meanRD2),mean(naboD2),mean(nstaD2),mean(nmumD2),mean(CoensBD2),
                    mean(meanRD3),mean(naboD3),mean(nstaD3),mean(nmumD3),mean(CoensBD3),
                    sce)

  nam<-paste("BW_SensSce_",sce,"_tmpRes.RData",sep="")
  save(sens_res,file=nam)

 }#close for loops for scenarios

